#include "Precompiled.h"
#include "Action.h"
#include "BaseNomad.h"
#include "Config.h"
#include "CellAttrCache.h"
#include "GridCellChoiceRenderer.h"
#include "FastComboEditor.h"


// c_strAction is the combox text of the action
const wxString c_strAction[Action::maxActionTypes] = {"Leave", 
	"Download to Jukebox", "Upload to PC", 
	"Delete", "Download Label", "Upload Label"};
const wxString c_strActionDone[Action::maxActionTypes] = {"", "Downloaded", 
	"Uploaded", "Deleted", "Label downloaded", "Label uploaded"};

// c_strChoices is a list of possible default actions depending on the type 
const wxString c_strChoices[c_nActionGroups][c_nActionChoices] = 
{	{c_strAction[Action::Delete], c_strAction[Action::PC2Nomad], c_strAction[Action::noAction]},
	{c_strAction[Action::Delete], c_strAction[Action::Nomad2PC], c_strAction[Action::noAction]},
	{c_strAction[Action::Label2Nomad], c_strAction[Action::Label2PC], c_strAction[Action::noAction]}
};


Action::ActionEnum GetActionEnum(const wxString& strAction)
{
	for (int n = 0; n < Action::maxActionTypes; n++)
	{
		if (strAction == c_strAction[n]) 
		{
			return (Action::ActionEnum)n;
		}
	}
	return Action::noAction;
}

Action::Action(const Config& config) : 
	m_action(noAction), m_bDone(false), 
	m_bWaitForCompletion(false), m_config(config),
	m_bUserModified(false), m_pAddToPlaylist(NULL), 
	m_pAttr(NULL), m_bEvenPair(false), m_bExecuting(false), m_nChoices(0)
{
}

Action::~Action()
{
	if (m_pAttr) 
	{
		m_pAttr->DecRef();
	}
}

// returns true if the other row in a pair item needs to be refreshed
bool Action::SetAction(const wxString& strAction)
{
	if (m_bDone) return false;
	for (int n = 0; n < m_nChoices; n++)
	{
		if (strAction == m_strChoices[n]) 
		{
			if (m_action == m_enumChoices[n]) return false;
			return SetAction(m_enumChoices[n]);
		}
	}
	// if strAction isn't a valid action, don't do anything
	return false;
}

const wxString& Action::GetActionStr() const
{
	return c_strAction[m_action];
}


bool Action::MoveToReCycleBin(const wxString& strFile)
{
#ifdef __WINDOWS__
	SHFILEOPSTRUCT fileop;
	fileop.hwnd = NULL; 
	fileop.wFunc = FO_DELETE;
	wxString strToDelete(strFile);
	strToDelete += '\0';
	fileop.pFrom = strToDelete;
	fileop.pTo = NULL;
	fileop.fFlags = FOF_ALLOWUNDO | FOF_SILENT | FOF_NOERRORUI |
		FOF_NOCONFIRMATION;
	return SHFileOperation(&fileop) == 0;
#else
	int nRet; 
	wxFileName fnTo;
	fnTo.AssignDir(m_config.GetDir(c_nTrashDir));
	wxFileName fnFrom(strFile);
	fnTo.SetFullName(fnFrom.GetFullName());
	wxLogNull logNo;				
	nRet = wxRenameFile(fnFrom.GetFullPath(), fnTo.GetFullPath());
	return nRet;
#endif
}
