#ifndef _ACTION_H_INCLUDED_
#define _ACTION_H_INCLUDED_

class BaseNomad;
class Config;
class wxGridCellAttr;
class CellAttrCache;
class Track;


const int c_nDefMissingOnNomad = 0;
const int c_nDefMissingOnPC = 1;
const int c_nDefPair = 2;
const int c_nActionGroups = 3;
const int c_nActionChoices = 3;
const int c_nMaxChoices = 5;

extern const wxString c_strAction[];
extern const wxString c_strActionDone[];
extern const wxString c_strChoices[c_nActionGroups][c_nActionChoices];


// abstract base class that handles actions on items
class Action
{
public:
	Action(const Config& config);
	virtual ~Action();
	enum ActionEnum
	{
		noAction = 0,
		PC2Nomad,
		Nomad2PC,
		Delete,
		Label2Nomad,
		Label2PC,
		maxActionTypes
	};
	ActionEnum GetAction() const
	{
		return m_action;
	}
	bool SetAction(const wxString& strAction);
	virtual bool SetAction(ActionEnum action) = 0;
	const wxString& GetActionStr() const;
	virtual bool DoAction(BaseNomad* pNomad) = 0;
	virtual bool DoPostAction(unsigned int nItemID) = 0;
	virtual int GetTimeNeeded() const = 0;
	virtual wxString GenErrorMsg() const = 0;
	virtual wxString GetDisplayStr() const = 0;
	virtual Action* CheckSplit() = 0;
	virtual bool MergeActions(Action* pNextAction) = 0;
	virtual wxString GetValue(int nCol, int nTrackIX) const = 0;
	virtual wxGridCellAttr* GetAttr(int nCol, int nTrackIX, 
		CellAttrCache& attrCache) const = 0;
	virtual void SetDefault() = 0; 
	virtual bool ColIsLessThan(int nCol, const Action* pAction) const = 0;
	const wxString& GetErrorStr() const
	{
		return m_strError;
	}
	bool IsDone() const
	{
		return m_bDone;
	}
	void SetErrorStr(const wxString& strError)
	{
		m_strError = strError;
		m_bWaitForCompletion = false;
	}
	bool HadError() const
	{
		return !m_strError.IsEmpty();
	}
	bool IsPending() const
	{
		return !m_bDone && !HadError() && m_action != noAction;
	}
	bool GetWaitForCompletion() const
	{
		return m_bWaitForCompletion && !m_bDone && !HadError()
			&& m_action != noAction;
	}
	Track* GetAddToPlaylist() const
	{
		return m_pAddToPlaylist;
	}
	void SetAddToPlaylist(Track* pAddToPlaylist)
	{
		m_pAddToPlaylist = pAddToPlaylist;
	}
	bool IsSecond() const
	{
		return (GetDependAction() && 
			(m_action == PC2Nomad || m_action == Nomad2PC));
	}
	virtual void SetDependAction(Action* pDependAction) = 0;
	virtual Action* GetDependAction() const = 0;
	void SetEvenPair(bool bEvenPair)
	{
		m_bEvenPair = bEvenPair;
		// avoid recursion, don't call SetEvenPair!!
		if (GetDependAction()) GetDependAction()->m_bEvenPair = bEvenPair; 
	}
	bool GetEvenPair() const
	{
		return m_bEvenPair;
	}
	int GetTypeOrder() const;	
	int GetColourIndex() const
	{
		if (IsPaired()) return 1+m_bEvenPair;
		return 0;
	}
	virtual bool IsPaired() const = 0;
	virtual bool IsLabelAction() const = 0;
	void SetExecuting(bool bExecuting)
	{
		m_bExecuting = bExecuting;
	}
protected:
	bool MoveToReCycleBin(const wxString& strFile);
	void SetChoice(int nIndex, ActionEnum action)
	{
		m_strChoices[nIndex] = c_strAction[action];
		m_enumChoices[nIndex] = action;
	}
	// m_fileDownLoaded remebers the file downloaded, so that it can add the tag later
	wxFileName m_fileDownLoaded; 
	ActionEnum m_action;
	wxString m_strError;		// when action failed this will have the error message 
	bool m_bDone;
	bool m_bWaitForCompletion;	// only used inconjuction with windows driver
	const Config& m_config;
	bool m_bUserModified;
	Track* m_pAddToPlaylist;
	mutable wxGridCellAttr* m_pAttr;
	bool m_bEvenPair;			// used to give the actions alternative colours
	bool m_bExecuting;
	wxString m_strChoices[c_nMaxChoices];
	int m_nChoices;
	ActionEnum m_enumChoices[c_nMaxChoices];
};

Action::ActionEnum GetActionEnum(const wxString& strAction);



#endif // _ACTION_H_INCLUDED_
