#ifndef _VIRTUALTABLE_H_INCLUDED_
#define _VIRTUALTABLE_H_INCLUDED_

#include "wx/grid.h"
#include "CellAttrCache.h"

using namespace std;
//class Action;
#include "Action.h"

typedef vector<Action*> ActionList;

enum ActionTableMode
{
	modeTrack,
	modePlaylist
};

// class ActionTable: stores the data of the grid
class ActionTable : public wxGridTableBase
{
public:
	ActionTable();
	~ActionTable();
	virtual int GetNumberCols();
	virtual int GetNumberRows()
	{
		return m_nNumRows;
	}
	virtual bool IsEmptyCell(int nRow, int nCol) { return false; }
	virtual wxString GetValue(int nRow, int nCol);
	virtual void SetValue(int nRow, int nCol, const wxString& value);
	virtual wxString GetColLabelValue(int nCol);
	virtual wxString GetRowLabelValue(int nRow);
	virtual wxGridCellAttr *GetAttr(int nRow, int nCol, 
		wxGridCellAttr::wxAttrKind kind);
	void AddAction(Action* pAction);
	void RemoveActions();
	void SetDefaultActions();
	void GetActionsInOrder(ActionList& listAction) const;
	int ActionsToDo() const;
	void Sort(int nCol);
	void SetGrid(wxGrid* pGrid)
	{
		m_pGrid = pGrid;
	}
	int ScrollToAction(const Action* pAction);
	void ForceRefresh()
	{
		m_pGrid->ForceRefresh();
	}
	ActionTableMode GetMode() const
	{
		return m_mode;
	}
	void SetMode(ActionTableMode mode)
	{
		m_mode = mode;
	}
private:
	void ShellSort(Action** pListAction, int nListSize, int nCol);
	bool SetSingleValue(int nRow, int nCol, const wxString& strValue);
	// this is the master copy of the actions
	// indexed by row (the second row of a label action will not be entered)
	map<int, Action*> m_mapAction;	
	CellAttrCache m_attrCache;		// similiar CellAttr's are re-used
	int m_nSortCol;		// the column which is currently sorrted
	bool m_bSortColDec;		// true if sort col is in desending order
	// need access to grid because it needs to be refreshed after certain actions
	wxGrid* m_pGrid;	
	int m_nNumRows;		// number of rows in grid (more than m_mapAction.size()!!)
	ActionTableMode m_mode;
};





#endif // _VIRTUALTABLE_H_INCLUDED_
