#include "Precompiled.h"
#include "BaseNomad.h"
#include "Track.h"
#include "Playlist.h"
#include "Action.h"

const wxString c_nAttribName[c_nAttribNames] = { "", "TITLE", "ARTIST", "ALBUM", 
	"GENRE", "CODEC", "FILE SIZE", "LENGTH", "TRACK NUM", "YEAR"};

BaseNomad::~BaseNomad()
{
	RemovePlaylists();
	RemoveTracks();
}

void BaseNomad::RemovePlaylists()
{
	PlaylistMap::iterator iterMapPlaylist; 
	for (iterMapPlaylist = m_mapPlaylist.begin(); 
		iterMapPlaylist != m_mapPlaylist.end(); iterMapPlaylist++) 
		delete iterMapPlaylist->second;
	m_mapPlaylist.clear();
}

void BaseNomad::RemoveTracks()
{
	map<unsigned int, Track*>::iterator iterMap; 
	for (iterMap = m_mapTrack.begin(); iterMap != m_mapTrack.end(); iterMap++) 
		delete iterMap->second;
	m_mapTrack.clear();
}

void BaseNomad::AddTrack(Track* pTrack)
{
	map<unsigned int, Track*>::iterator iterMap;
	iterMap = m_mapTrack.find(pTrack->GetItemID()); 
	if (iterMap != m_mapTrack.end()) 
	{
		wxLogMessage("Track with duplicate track ID: %s", (const char*)
			pTrack->GetDisplayName());
	}
	else m_mapTrack[pTrack->GetItemID()] = pTrack;
}

void BaseNomad::SetTrackAttrib(Track* pTrack, const char* szAttribName, 
	wxString strValue, int nValue) const
{
	for (int n = 1; n < c_nAttribNames; n++)
	{
		if (c_nAttribName[n] == szAttribName)
		{
			switch (n)
			{
				case taGenre:
				case taTitle:
				case taAlbum:
				case taArtist:
				case taCodec:
					pTrack->SetAttrib((trackAttribStr)n, strValue);
					break;
				case taFileSize:
					pTrack->SetFileSize(nValue);
					break;
				case taLength:
					pTrack->SetTrackLength(nValue);
					break;
				case taTrackNum:
					pTrack->SetTrackNum(nValue);
					break;
				case taYear:
					pTrack->SetYear(nValue);
				default:
					break;
			}	// end switch
		return;
		}	// end if
	} // end for
	wxLogMessage("Unknown Label %s: %s", szAttribName, 
		(const char*)strValue);
	wxLogMessage(pTrack->GetDisplayName());
}

void BaseNomad::LoadPlaylists()
{
//	if (m_bPlaylistsRead) return;
	RemovePlaylists();
	Playlist* pPlaylist;
	map<wxString, Playlist*>::iterator iterMap; 
	pPlaylist = GetFirstPlaylist();
	wxString strLowerName;
	while (pPlaylist)
	{
		strLowerName = pPlaylist->GetName().Lower();
		iterMap = m_mapPlaylist.find(strLowerName); 
		if (iterMap != m_mapPlaylist.end()) 
		{
			wxLogMessage("Panic! duplicate playlist: %s", (const char*)
				pPlaylist->GetName());
			delete pPlaylist;
		}
		else 
		{
			m_mapPlaylist[strLowerName] = pPlaylist;
		}
		pPlaylist = GetNextPlaylist();
	}
	if (m_nLastError != 0) return;
	GetPlaylistTracks();
	m_bPlaylistsRead = true;
}	//	LoadPlaylists


wxString* BaseNomad::GetPlaylistNames() const
{
	wxArrayString listPlaylist;
	map<wxString, Playlist*>::const_iterator iterMapPlaylist; 
	for (iterMapPlaylist = m_mapPlaylist.begin(); 
		iterMapPlaylist != m_mapPlaylist.end(); iterMapPlaylist++) 
		listPlaylist.Add(iterMapPlaylist->second->GetName());
	return listPlaylist.GetStringArray();
}

bool BaseNomad::AddTracksToPlaylist(const wxString& strPlaylistName, 
	const ActionList& listAction)
{
	if (strPlaylistName.IsEmpty()) return true;
	int nToAdd = 0;
	unsigned int n;
	for (n = 0; n < listAction.size(); n++)
	{
		if (listAction[n]->GetAddToPlaylist())
		{
			nToAdd++;
		}
	}
	if (nToAdd == 0) return true;
	map<wxString, Playlist*>::iterator iterMap; 
	iterMap = m_mapPlaylist.find(strPlaylistName.Lower()); 
	Playlist* pPlaylist;
	if (iterMap == m_mapPlaylist.end()) 
	{
		pPlaylist = new Playlist;
		pPlaylist->SetName(strPlaylistName);
	}
	else pPlaylist = iterMap->second;
	for (n = 0; n < listAction.size(); n++)
	{
		if (listAction[n]->GetAddToPlaylist())
		{
			pPlaylist->AddNoDups(listAction[n]->GetAddToPlaylist());
			listAction[n]->SetAddToPlaylist(NULL);
		}
	}
	return CreatePlaylist(pPlaylist);
}	// AddTracksToPlaylist


bool BaseNomad::CreatePlaylist(Playlist* pPlaylist)
{
	wxString strName(pPlaylist->GetName().Lower());
	if (SendPlaylist(pPlaylist))
	{
		m_mapPlaylist[strName] = pPlaylist;
	}
	else
	{
		// if there is an error then delete the playlist, unless
		// it's already in the map
		map<wxString, Playlist*>::iterator iterMap; 
		iterMap = m_mapPlaylist.find(strName); 
		if (iterMap == m_mapPlaylist.end() || iterMap->second != pPlaylist)
		{
			delete pPlaylist;
		}
		return false;
	}
	return true;
}	// CreatePlaylist

bool BaseNomad::DeletePlaylist(const Playlist* pPlaylist)
{
	wxString strName(pPlaylist->GetName().Lower());
	Playlist* pPlaylist2;
	if (DeleteItem(pPlaylist))
	{
		map<wxString, Playlist*>::iterator iterMap; 
		iterMap = m_mapPlaylist.find(strName); 
		if (iterMap != m_mapPlaylist.end())
		{
			pPlaylist2 = iterMap->second;
			delete iterMap->second;
			m_mapPlaylist.erase(iterMap);
		}
		return true;
	}
	return false;
}
