#ifndef _BASENOMAD_H_INCLUDED_
#define _BASENOMAD_H_INCLUDED_


class Track;
class Playlist;
class Action;

using namespace std;

typedef map<wxString, Playlist*> PlaylistMap;
typedef vector<Action*> ActionList;

const int c_nAttribNames = 10;
extern const wxString c_nAttribName[];

class BaseNomad
{
public:
	BaseNomad() : m_nLastError(0), m_bPlaylistsRead(false){}
	virtual ~BaseNomad();
	void RemoveTracks();
	void RemovePlaylists();
	virtual bool InitJukeBox() = 0;
	virtual bool StartLoadAllTracks() = 0;
	virtual bool MoveNextTrack() = 0;
	virtual Track* LoadTrackData() = 0;
	virtual bool SendItem(Track* pTrack) = 0;
	virtual bool GetItem(const Track* pTrack, wxFileName& fileSpec) = 0;
	virtual bool DeleteItem(const Track* pTrack) = 0;
	virtual bool DeleteItem(const Playlist* pPlaylist) = 0;
	virtual bool SendLabel(const Track* pTrackPC, 
		const Track* pTrackNomad) = 0;
	virtual void CancelTransfer() = 0;
	virtual wxString GetLastErrorStr() const = 0;
	virtual Playlist* GetFirstPlaylist() = 0;
	virtual Playlist* GetNextPlaylist() = 0;
	virtual bool SendPlaylist(Playlist* pPlaylist) = 0;
	virtual bool GetPlaylistTracks() = 0;	// to do: rename this to match??
	bool CreatePlaylist(Playlist* pPlaylist);
	bool DeletePlaylist(const Playlist* pPlaylist);
	bool HadError() const
	{
		return m_nLastError != 0;	
	}
	void ClearError()
	{
		m_nLastError = 0;	
	}
	void LoadPlaylists();
	wxString* GetPlaylistNames() const;
	int GetCountPlaylist() const
	{
		return m_mapPlaylist.size();
	}
	bool AddTracksToPlaylist(const wxString& strPlaylistName, 
		const ActionList& listAction);
	bool GetPlaylistsRead() const
	{
		return m_bPlaylistsRead;
	}
	const PlaylistMap& GetMapPlaylist() const
	{
		return m_mapPlaylist;
	}
protected:
	void SetTrackAttrib(Track* pTrack, const char* szLabel, 
		wxString strValue, int nValue) const;
	void AddTrack(Track* pTrack);
	int m_nLastError;
	mutable wxString m_strLastError;
	PlaylistMap m_mapPlaylist;
	map<unsigned int, Track*> m_mapTrack;
	bool m_bPlaylistsRead;
};





#endif // _BASENOMAD_H_INCLUDED_
