#include "Precompiled.h"
#include "CellAttrCache.h"
#include "wx/grid.h"
#include "GridCellIconRenderer.h"


CellAttrCache::CellAttrCache()
{
	const int c_nShade = 32;
	for (int n = 0; n < ctMaxCellTypesUsed; n++)
		m_pAttr[n] = NULL;
	m_colourBack[0] = wxSystemSettings::GetSystemColour(wxSYS_COLOUR_BTNFACE);
	unsigned char nRed = m_colourBack[0].Red();
	unsigned char nGreen = m_colourBack[0].Green();
	unsigned char nBlue = m_colourBack[0].Blue();
	// make it lighter than RGB
	if (nBlue >= 256 - c_nShade)
	{
		m_colourBack[1] = wxColour(nRed, nGreen, (unsigned char)(nBlue-c_nShade/2));
		if (nRed > c_nShade/2) nRed -= c_nShade/2;
		if (nGreen > c_nShade/2) nGreen -= c_nShade/2;
		nBlue -= c_nShade;
		m_colourBack[2] = wxColour(nRed, nGreen, nBlue);
	}
	else
	{
		if (nBlue <= 256 - c_nShade/2) nBlue += c_nShade/2;
		m_colourBack[2] = wxColour(nRed, nGreen, nBlue);
		if (nRed <= 256 - c_nShade/2) nRed += c_nShade/2;
		if (nGreen <= 256 - c_nShade/2) nGreen += c_nShade/2;
		if (nBlue <= 256 - c_nShade/2) nBlue += c_nShade/2;
		m_colourBack[1] = wxColour(nRed, nGreen, nBlue);
	}
//	m_colourBack[1] = wxColour(208, 208, 224);
//	m_colourBack[2] = wxColour(192, 192, 208);
}

CellAttrCache::~CellAttrCache()
{
	for (int n = 0; n < ctMaxCellTypesUsed; n++)
		if (m_pAttr[n]) m_pAttr[n]->DecRef();
}

wxGridCellAttr* CellAttrCache::Get(CellTypeUsed cellType)
{
	if (m_pAttr[cellType] == NULL)
	{
		m_pAttr[cellType] = new wxGridCellAttr;
		m_pAttr[cellType]->SetReadOnly(TRUE);
		m_pAttr[cellType]->SetOverflow(FALSE);
		switch (cellType)
		{
			case ctIcon:
				m_pAttr[cellType]->SetRenderer(new wxGridCellIconRenderer);
				break;
			case ctIconLight:
				m_pAttr[cellType]->SetBackgroundColour(m_colourBack[1]); 
				m_pAttr[cellType]->SetRenderer(new wxGridCellIconRenderer);
				break;
			case ctIconDark:
				m_pAttr[cellType]->SetBackgroundColour(m_colourBack[2]); 
				m_pAttr[cellType]->SetRenderer(new wxGridCellIconRenderer);
				break;
			case ctNormal:
				break;
			case ctPairLight:
				m_pAttr[cellType]->SetBackgroundColour(m_colourBack[1]); 
				break;
			case ctPairDark:
				m_pAttr[cellType]->SetBackgroundColour(m_colourBack[2]); 
				break;
			default:
				wxASSERT(FALSE);
		}
	}
	m_pAttr[cellType]->IncRef();
	return m_pAttr[cellType];
}

