#include "Precompiled.h"
#include "CommonFunc.h"

using namespace std;

wxString StripDangerous(const wxString& strValue)
{
	wxString str = strValue;
	for (unsigned int n = 0; n < str.Length(); n++)
	{
		if (!wxIsalnum(str.GetChar(n)) && str.GetChar(n) != ' ' && 
			str.GetChar(n) != '[' && str.GetChar(n) != ']' &&
			str.GetChar(n) != '(' && str.GetChar(n) != ')' &&
			str.GetChar(n) != '-' && str.GetChar(n) != '_')
		 str.GetWritableChar(n) = '_';
	}
	return str;
}

// The edit distance returns the amount of differences, then divide by the 
// string length. 
// For instance a weighted edit distance of 1.0 means all the characters are different!
double WeighedEditDistance(const wxString& strA, const wxString& strB)
{
	int nLenA = strA.Length();
	int nLenB = strB.Length();
	if (nLenA == 0 || nLenB == 0) return false;
	int nMaxLen = nLenA;
	if (nLenB > nLenA) nMaxLen = nLenB;
	int* pnMatrix = new int[(nLenA+1)*(nLenB+1)];
	int nRow, nCol;
	for (nRow = 0; nRow <= nLenA; nRow++)
	{
		pnMatrix[nRow*nLenB + 0] = nRow; 
	}
	for (nCol = 0; nCol <= nLenB; nCol++)
	{
		pnMatrix[nCol] = nCol; 
	}
	int nCost, nValue;
	for (nCol = 1; nCol <= nLenB; nCol++)
	{
		for (nRow = 1; nRow <= nLenA; nRow++)
		{
			if (strA.GetChar(nRow-1) == strB.GetChar(nCol-1)) nCost = 0;
			else nCost = 1;
			nValue = min(pnMatrix[(nRow-1)*nLenB + nCol]+1, 
				pnMatrix[nRow*nLenB + nCol-1]+1);
			nValue = min(nValue, pnMatrix[(nRow-1)*nLenB + nCol-1] + nCost);
			pnMatrix[nRow*nLenB + nCol] = nValue;
		}
	}
	int nResult = pnMatrix[nLenA*nLenB +nLenB];
	delete [] pnMatrix;
//	wxLogDebug(strA);
//	wxLogDebug(strB);
	if (nResult == 0) return (double)nMaxLen; // safty check
//	wxLogDebug("Edit Distance: %d, weighted = %f", nResult, (double)nMaxLen / nResult);
	return (double)nMaxLen / nResult;
}	// WithinEditDistance

wxIconButton::wxIconButton(wxWindow *parent, wxWindowID id, 
	const wxIcon& icon, wxFont& font,
	const wxPoint& pos, const wxSize& size,
	long style, const wxValidator& validator,
	const wxString& name) 
{
	wxBitmap bitmap(size.GetWidth()-8, size.GetHeight()-8);
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	dc.SetFont(font);
	dc.SetBackground(wxBrush(
		wxSystemSettings::GetSystemColour(wxSYS_COLOUR_BTNFACE), wxSOLID));
	dc.Clear();
	dc.DrawIcon(icon, 4, 4);
	dc.DrawText(name, icon.GetWidth()+4, 4);
	dc.SelectObject( wxNullBitmap );
	Create(parent, id, bitmap, pos, size, style, validator, name);
}
