#include "Precompiled.h"
#include "Config.h"
#include <wx/config.h>


Config::Config() : m_bChangedDefault(false), m_nLastPage(0), m_bEnableLog(false)
{
	m_actionDefault[c_nDefMissingOnNomad] = Action::noAction;
	m_actionDefault[c_nDefMissingOnPC] = Action::noAction;
	m_actionDefault[c_nDefPair] = Action::noAction;
	wxConfigBase *cfg = wxConfigBase::Get();
	wxString str, strTemp;
	int n;
	for (n = 0; n <= c_nLastCol; n++)
	{
		m_bShowCol[n] = true;
	}
	for (n = 0; n < c_nMaxDirs; n++)
	{
		str.Printf("dir%d", n);
		cfg->Read(str, &strTemp);
		SetDir(n, strTemp);
	}
	for (n = c_nDefMissingOnNomad; n <= c_nDefPair; n++)
	{
		str.Printf("defaultAction%d", n);
		cfg->Read(str, (long*)&m_actionDefault[n]);
	}
	if (m_strDir[c_nTrashDir].IsEmpty() && wxGetHomeDir())
	{
		// KDE trash folder
		wxFileName fn;
		fn.AssignDir(wxGetHomeDir());
		fn.AppendDir("Desktop");
		fn.AppendDir("Trash");
		if (fn.DirExists())
		{
			m_strDir[c_nTrashDir] = fn.GetFullPath();
		}
		else	// other possible KDE trash
		{
			fn.AssignDir(wxGetHomeDir());
			fn.AppendDir("Desktop");
			fn.AppendDir("Wastebin");
			if (fn.DirExists())
			{
				m_strDir[c_nTrashDir] = fn.GetFullPath();
			}
			else
			{
				// gnome trash folder
				fn.AssignDir(wxGetHomeDir());
				fn.AppendDir(".Trash");
				if (fn.DirExists())
				{
					m_strDir[c_nTrashDir] = fn.GetFullPath();
				}
			}
		}
	}
	cfg->Read("Download_playlist", &m_strDownloadPlaylist);
	cfg->Read("MP3_Player", &m_strLinuxMP3Command);
	for (n = 0; n < c_nMaxCompareExtra; n++)
	{
		m_bCompareExtra[n] = false;
		str.Printf("ComapareExtra%d", n);
		cfg->Read(str, (long*)&m_bCompareExtra[n]);
	}
}	// end contructor


Config::~Config()
{
	wxConfigBase *cfg = wxConfigBase::Get();
	wxString str;
	int n;
	for (n = 0; n < c_nMaxDirs; n++)
	{
		str.Printf("dir%d", n);
		cfg->Write(str, m_strDir[n]);
	}
	for (n = c_nDefMissingOnNomad; n <= c_nDefPair; n++)
	{
		str.Printf("defaultAction%d", n);
		cfg->Write(str, (long)m_actionDefault[n]);
	}
	cfg->Write("Download_playlist", m_strDownloadPlaylist);
#ifndef __WINDOWS__
	cfg->Write("MP3_Player", m_strLinuxMP3Command);
#endif
	for (n = 0; n < c_nMaxCompareExtra; n++)
	{
		str.Printf("ComapareExtra%d", n);
		cfg->Write(str, (long)m_bCompareExtra[n]);
	}
}

bool Config::IsExcluded(const wxString& strDir) const
{
	for (int n = c_nFirstExcludeDir; n <= c_nLastExcludeDir; n++)
	{
		if (!m_strDir[n].IsEmpty() && strDir.StartsWith(m_strDirNoSlash[n])) return true;
	}
	return false;
}

void Config::RemoveIncludeDir(wxString& strFileSpec) const
{
	for (int n = c_nFirstIncludeDir; n <= c_nLastIncludeDir; n++)
	{
		if (!m_strDir[n].IsEmpty() && strFileSpec.StartsWith(m_strDir[n], &strFileSpec)) return;
	}
}

bool Config::IsIncludeDirSet() const
{
	for (int n = c_nFirstIncludeDir; n <= c_nLastIncludeDir; n++)
	{
		if (!m_strDir[n].IsEmpty()) return true;
	}
	return false;
}


const wxString& Config::GetUploadDir() const
{
	if (!m_strDir[c_nUploadDir].IsEmpty()) return m_strDir[c_nUploadDir];
	for (int n = c_nFirstIncludeDir; n <= c_nLastIncludeDir; n++)
	{
		if (!m_strDir[n].IsEmpty()) return m_strDir[n];
	}
	return m_strDir[0];
}

bool Config::SetDir(int n, const wxString& strDir)
{
	wxASSERT(n >= 0 && n < c_nMaxDirs);
	wxFileName fn(strDir);
	if (!strDir.IsEmpty() && !fn.DirExists()) return false;
	m_strDir[n] = strDir;
	m_strDirNoSlash[n] = strDir;
	if (strDir.IsEmpty()) return true;
	if (strDir.Last() != wxFILE_SEP_PATH) 
		m_strDir[n].Append(wxFILE_SEP_PATH);
	else
		m_strDirNoSlash[n].RemoveLast();
	return true;
}

bool Config::IsValidDir(int n) const
{
	wxFileName fn(m_strDir[n]);
	if (!m_strDir[n].IsEmpty() && fn.DirExists()) return true;
	return false;
}
