#ifndef _CONFIG_H_INCLUDED_
#define _CONFIG_H_INCLUDED_

const int c_nIncludeDirs = 3;
const int c_nExcludeDirs = 3;
const int c_nMaxDirs = c_nIncludeDirs + c_nExcludeDirs + 2;
const int c_nFirstIncludeDir = 0;
const int c_nLastIncludeDir = c_nIncludeDirs-1;
const int c_nFirstExcludeDir = c_nIncludeDirs;
const int c_nLastExcludeDir = c_nMaxDirs-3;
const int c_nUploadDir = c_nMaxDirs-2;
const int c_nTrashDir = c_nMaxDirs-1;

#include "Action.h"
#include "CommonFunc.h"

const int c_nMaxCompareExtra = 3;
// controls the user-defined configuration settings
class Config
{
public:
	Config();
	~Config();
	const wxString& GetDir(int n) const
	{
		wxASSERT(n >= 0 && n < c_nMaxDirs);
		return m_strDir[n];
	}
	bool SetDir(int n, const wxString& strDir);
	bool IsValidDir(int n) const;
	bool IsExcluded(const wxString& strDir) const;
	void RemoveIncludeDir(wxString& strFileSpec) const;
	bool IsIncludeDirSet() const;
	const wxString& GetUploadDir() const;
	Action::ActionEnum GetActionDefault(int n) const
	{
		return m_actionDefault[n];
	}
	void SetActionDefault(int n, Action::ActionEnum actionDefault)
	{
		if (m_actionDefault[n] != actionDefault)
		{
			m_actionDefault[n] = actionDefault;
			m_bChangedDefault = true;
		}
	}
	bool IsChangedDefault() const
	{
		return m_bChangedDefault;
	}
	void SetChangedDefault(bool bChangedDefault)
	{
		m_bChangedDefault = bChangedDefault;
	}
	int GetLastPage() const
	{
		return m_nLastPage;
	}
	void SetLastPage(int nLastPage)
	{
		m_nLastPage = nLastPage;
	}
	bool GetEnableLog() const
	{
		return m_bEnableLog;
	}
	void SetEnableLog(bool bEnableLog)
	{
		m_bEnableLog = bEnableLog;
	}
	const wxString& GetDownloadPlaylist() const
	{
		return m_strDownloadPlaylist;
	}
	void SetDownloadPlaylist(const wxString& strDownloadPlaylist)
	{
		m_strDownloadPlaylist = strDownloadPlaylist;
	}
	bool NeedPlaylists() const
	{
		return !m_strDownloadPlaylist.IsEmpty();
	}
	const wxString& GetLinuxMP3Command() const
	{
		return m_strLinuxMP3Command;
	}
	void SetLinuxMP3Command(const wxString& strLinuxMP3Command) 
	{
		m_strLinuxMP3Command = strLinuxMP3Command;
	}
	bool GetShowCol(int nIndex) const
	{
		return m_bShowCol[nIndex];
	}
	void SetShowCol(int nIndex, bool bShowCol)
	{
		m_bShowCol[nIndex] = bShowCol;
	}
	bool GetCompareExtra(int nIndex) const
	{
		return m_bCompareExtra[nIndex];
	}
	void SetCompareExtra(int nIndex, bool bCompareExtra)
	{
		m_bCompareExtra[nIndex] = bCompareExtra;
	}
private:
	wxString m_strDir[c_nMaxDirs];
	wxString m_strDirNoSlash[c_nMaxDirs];
	Action::ActionEnum m_actionDefault[c_nActionGroups];
	bool m_bChangedDefault;
	int m_nLastPage;
	bool m_bEnableLog;
	wxString m_strDownloadPlaylist;
	wxString m_strLinuxMP3Command;
	bool m_bCompareExtra[c_nMaxCompareExtra];
	bool m_bShowCol[c_nLastCol+1];
};





#endif // _CONFIG_H_INCLUDED_
