#include "Precompiled.h"
#include "ConfigDlg.h"
#include "EzMessageBox.h"
#include <wx/notebook.h>
#include "Action.h"
#include "BaseNomad.h"

const int IDC_OPEN_DIR_DIALOG = 1004;
const int IDC_DEFAULT_ACTION = 1020;

BEGIN_EVENT_TABLE(ConfigDlg, wxDialog)
	EVT_BUTTON(-1, ConfigDlg::OnButton)
END_EVENT_TABLE()

ConfigDlg::ConfigDlg(wxWindow *parent, Config* pConfig, const BaseNomad* pNomad)
	: wxDialog(parent, -1, wxString(_T("Configuration")), wxDefaultPosition, wxDefaultSize, 
		wxRAISED_BORDER|wxCAPTION|wxRESIZE_BORDER), 
		m_pNomad(pNomad), m_nDirsUsed(c_nMaxDirs-1)
{
	m_pConfig = pConfig;
	wxBoxSizer *psizerTop = new wxBoxSizer( wxVERTICAL );
	m_pNotebook = new wxNotebook(this, -1);
	wxNotebookSizer* psizerNotebook = new wxNotebookSizer(m_pNotebook);
	m_pNotebook->AddPage(MakeDirPanel(), "Directories");
	m_pNotebook->AddPage(MakeMiscPanel(), "Miscellaneous");
	m_pNotebook->SetSelection(m_pConfig->GetLastPage());


	psizerTop->Add(psizerNotebook, 1, wxEXPAND | wxALL, 4); 
	wxBoxSizer *psizerButton = new wxBoxSizer( wxHORIZONTAL );
	wxButton *pbtnCancel = new wxButton(this, wxID_CANCEL, _T("&Cancel"), wxPoint(-1, -1),
					   wxSize( 80, 29));
	m_pbtnOK = new wxButton(this, wxID_OK, _T("OK"), wxPoint(-1, -1),
					   wxSize( 80, 29));
	psizerButton->Add(m_pbtnOK, 0, wxALIGN_CENTER | wxALL, 5);
	psizerButton->Add(pbtnCancel, 0, wxALIGN_CENTER | wxALL, 5);
	psizerTop->Add(psizerButton, 0, wxALIGN_CENTER); 
	SetAutoLayout(TRUE);
	SetSizer(psizerTop);
	psizerTop->SetSizeHints(this);
	psizerTop->Fit(this);
	m_pbtnOK->SetFocus();
	m_pbtnOK->SetDefault();
	m_pConfig->SetChangedDefault(false);
}	// end constructor

wxPanel* ConfigDlg::MakeDirPanel()
{
	wxPanel* pPanel = new wxPanel(m_pNotebook, -1, wxDefaultPosition, wxDefaultSize,
	  wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);
	wxBoxSizer *psizerTopPanel = new wxBoxSizer( wxVERTICAL );
	psizerTopPanel->Add(0, 5, 0, wxGROW); // spacer
	wxStaticText* pStatic = new wxStaticText(pPanel, -1, _T("Directories to search"));
	psizerTopPanel->Add(pStatic, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
	wxFlexGridSizer *psizerGrid = new wxFlexGridSizer(c_nIncludeDirs, 2, 5, 5);
	int n;
	for (n = c_nFirstIncludeDir; n <= c_nLastIncludeDir; n++)
	{
		m_pDir[n] = new wxTextCtrl(pPanel, -1, m_pConfig->GetDir(n), 
			wxDefaultPosition, wxSize(400,-1));

		psizerGrid->Add(m_pDir[n],
			0,            // not vertically stretchable
			wxEXPAND |    // make horizontally stretchable
			wxALL,        //   and make border all around
			5);         // set border width to 5
		m_pbtnDirDialog[n] = new wxButton(pPanel, IDC_OPEN_DIR_DIALOG+n, _T("..."), 
			wxDefaultPosition, wxSize(20,-1));
		psizerGrid->Add(m_pbtnDirDialog[n],	0,	wxALL, 5);
	}
	m_pDir[0]->SetToolTip("Note: all sub-directories are included in the search");
	psizerTopPanel->Add(psizerGrid, 0, wxEXPAND); 

	psizerTopPanel->Add(0, 5, 0, wxGROW); // spacer
	psizerTopPanel->Add(new wxStaticText(pPanel, -1, "Directories to exclude from search"), 0,
		wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
	wxFlexGridSizer *psizerExclude = new wxFlexGridSizer(c_nExcludeDirs, 2, 5, 5);
	for (n = c_nFirstExcludeDir; n <= c_nLastExcludeDir; n++)
	{
		m_pDir[n] = new wxTextCtrl(pPanel, -1, m_pConfig->GetDir(n), wxDefaultPosition, 
			wxSize(400,-1));
		psizerExclude->Add(m_pDir[n],	0, wxEXPAND | wxALL, 5);         
		m_pbtnDirDialog[n] = new wxButton(pPanel, IDC_OPEN_DIR_DIALOG+n, _T("..."), 
			wxDefaultPosition, wxSize(20,-1));
		psizerExclude->Add(m_pbtnDirDialog[n],	0,	wxALL, 5);
	}
	psizerTopPanel->Add(psizerExclude,	0, wxEXPAND);
	// Upload directory
	psizerTopPanel->Add(0, 5, 0, wxGROW); // spacer
	psizerTopPanel->Add(new wxStaticText(pPanel, -1, "Upload Directory"), 0,
		wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
	wxFlexGridSizer *psizerUpload = new wxFlexGridSizer(1, 2, 5, 5);
	m_pDir[c_nUploadDir] = new wxTextCtrl(pPanel, -1, m_pConfig->GetDir(c_nUploadDir), 
		wxDefaultPosition, wxSize(400,-1));
	m_pDir[c_nUploadDir]->SetToolTip("Tracks and playlists are uploaded to this directory"); 
	psizerUpload->Add(m_pDir[c_nUploadDir],	0, wxEXPAND|wxALL, 5);         
	m_pbtnDirDialog[c_nUploadDir] = new wxButton(pPanel, 
		IDC_OPEN_DIR_DIALOG+c_nUploadDir, _T("..."), wxDefaultPosition, wxSize(20,-1));
	psizerUpload->Add(m_pbtnDirDialog[c_nUploadDir], 0, wxALL, 5);
	psizerTopPanel->Add(psizerUpload, 0, wxEXPAND);

#ifndef __WINDOWS__
	psizerTopPanel->Add(0, 5, 0, wxGROW); // spacer
	psizerTopPanel->Add(new wxStaticText(pPanel, -1, _T("Trash folder")), 0,
		wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
	wxFlexGridSizer *psizerTrash = new wxFlexGridSizer(1, 2, 5, 5);
	m_pDir[c_nTrashDir] = new wxTextCtrl(pPanel, -1, m_pConfig->GetDir(c_nTrashDir), 
		wxDefaultPosition, wxSize(400,-1));
	psizerTrash->Add(m_pDir[c_nTrashDir], 0, wxEXPAND|wxALL, 5); 
	m_pDir[c_nTrashDir]->SetToolTip("Note: If this is left blank then no tracks can be deleted from the PC");        
	m_pbtnDirDialog[c_nTrashDir] = new wxButton(pPanel, 
		IDC_OPEN_DIR_DIALOG+c_nTrashDir, _T("..."), 
		wxDefaultPosition, wxSize(20,-1));
	psizerTrash->Add(m_pbtnDirDialog[c_nTrashDir], 0, wxALL, 5);
	psizerTopPanel->Add(psizerTrash,	0, wxEXPAND);
	m_nDirsUsed = c_nMaxDirs;
#endif

	pPanel->SetSizer(psizerTopPanel);
	pPanel->SetAutoLayout( TRUE );
	psizerTopPanel->Fit(pPanel);
	return pPanel;
}


const wxString c_strDefaultLabel[c_nActionGroups] = 
	{"For PC items not found on Jukebox:", "For Jukebox items not found on PC:", 
	"For Similiar items:"};

const wxString c_strCompareExtra[c_nMaxCompareExtra] = {"Genre", "Track number", "Year"};

wxPanel* ConfigDlg::MakeMiscPanel()
{
	wxPanel* pPanel = new wxPanel(m_pNotebook, -1, wxDefaultPosition, wxDefaultSize,
	  wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);
	wxBoxSizer *psizerTopPanel = new wxBoxSizer( wxVERTICAL );
	psizerTopPanel->Add(0, 5, 0, wxGROW); // spacer
	wxStaticBox* pStaticBox = new wxStaticBox(pPanel, -1, "Default actions");
	wxStaticBoxSizer *outlineSizer = new wxStaticBoxSizer(pStaticBox, wxVERTICAL);
	wxFlexGridSizer *psizerGrid = new wxFlexGridSizer(c_nDefPair, 2, 5, 5);
	int n;
	for (n = 0; n <= c_nDefPair; n++)
	{
		psizerGrid->Add(new wxStaticText(pPanel, -1, c_strDefaultLabel[n]), 0,
			wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
		m_pDefAction[n] = new wxComboBox(pPanel, IDC_DEFAULT_ACTION, 
			c_strAction[m_pConfig->GetActionDefault(n)],
			wxDefaultPosition, wxSize(150,-1),
			c_nActionChoices, c_strChoices[n], wxCB_READONLY);
		psizerGrid->Add(m_pDefAction[n],	0,	wxALL, 5);
	}
	m_pDefAction[c_nDefPair]->SetToolTip("Also applies to tracks with with different file sizes"); 
	outlineSizer->Add(psizerGrid, 0, wxEXPAND|wxALL, 5); 
	psizerTopPanel->Add(outlineSizer, 0, wxEXPAND|wxALL, 5); 
	
	// add new tracks playlist
	wxFlexGridSizer *psizerGrid2 = new wxFlexGridSizer(2, 2, 5, 5);
	psizerGrid2->Add(new wxStaticText(pPanel, -1, 
		"Automatically add downloaded tracks\n to this playlist:"), 
		0,	wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);

	wxString* pString = m_pNomad->GetPlaylistNames();
	m_pDownloadPlaylist = new wxComboBox(pPanel, -1, m_pConfig->GetDownloadPlaylist(),
			wxDefaultPosition, wxSize(150,-1),
			m_pNomad->GetCountPlaylist(), pString, TRUE);
	if (pString) delete [] pString;
	psizerGrid2->Add(m_pDownloadPlaylist, 0, wxALL, 5);
	m_pDownloadPlaylist->SetToolTip("Note: Leaving this blank disables feature.");
#ifndef __WINDOWS__
	psizerGrid2->Add(new wxStaticText(pPanel, -1, 
		"MP3 player command (e.g. xmms):"), 
		0,	wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
	m_pMP3Command = new wxTextCtrl(pPanel, -1, m_pConfig->GetLinuxMP3Command(), 
		wxDefaultPosition, wxSize(150,-1));
	psizerGrid2->Add(m_pMP3Command, 0, wxALL, 5);
//	m_pMP3Command->SetToolTip("Used to play tracks from the main window
#endif

	wxStaticBox* pStaticBox2 = new wxStaticBox(pPanel, -1, "Compare extra fields when matching tracks");
	wxStaticBoxSizer *outlineSizer2 = new wxStaticBoxSizer(pStaticBox2, wxVERTICAL);
	wxFlexGridSizer *psizerCompareExtra = new wxFlexGridSizer(1, c_nMaxCompareExtra, 5, 5);
	for (n = 0; n < c_nMaxCompareExtra; n++)
	{
		m_pChkCompareExtra[n] = new wxCheckBox(pPanel, -1, 
			c_strCompareExtra[n]);
		m_pChkCompareExtra[n]->SetValue(m_pConfig->GetCompareExtra(n));
		psizerCompareExtra->Add(m_pChkCompareExtra[n], 0, wxALL, 5);
	}
	outlineSizer2->Add(psizerCompareExtra, 0, wxEXPAND|wxALL, 5); 
	psizerTopPanel->Add(outlineSizer2, 0, wxEXPAND|wxALL, 5); 

	m_pChkEnableLog = new wxCheckBox(pPanel, -1, 
		"Enable logging");
	m_pChkEnableLog->SetToolTip("Diagnostic data will be save to a file called log.txt in the upload directory");
	m_pChkEnableLog->SetValue(m_pConfig->GetEnableLog());
	psizerTopPanel->Add(psizerGrid2, 0, wxEXPAND|wxALL, 5); 
	psizerTopPanel->Add(m_pChkEnableLog, 0,
			wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
	pPanel->SetSizer(psizerTopPanel);
	pPanel->SetAutoLayout( TRUE );
	psizerTopPanel->Fit(pPanel);
	return pPanel;
}

// We have one event handler for all button-click events
void ConfigDlg::OnButton(wxCommandEvent& event)
{
	int n;
	m_pConfig->SetLastPage(m_pNotebook->GetSelection());
	if (event.GetId() == wxID_OK)
	{
#ifndef __WINDOWS__
		if (m_pDir[c_nTrashDir]->GetValue().IsEmpty())
		{
			wxString str("When a track is deleted it is moved to the trash folder\n");
			str += "Therefore you must enter a thrash folder";
			EzMessageBox("Error", str, 0, "OK");
			return;
		}
		m_pConfig->SetLinuxMP3Command(m_pMP3Command->GetValue());
#endif
		for (n = 0; n < m_nDirsUsed; n++)
		{
			if (!m_pConfig->SetDir(n, m_pDir[n]->GetValue()))
			{
				wxString str("Invalid directory: ");
				str += m_pDir[n]->GetValue();
				EzMessageBox("Error", str, 0, "OK");
				return;
			}
		}
		for (n = 0; n <= c_nDefPair; n++)
		{
			m_pConfig->SetActionDefault(n, GetActionEnum(m_pDefAction[n]->GetValue()));
		}
		m_pConfig->SetEnableLog(m_pChkEnableLog->GetValue());
		
		for (n = 0; n < c_nMaxCompareExtra; n++)
		{
			m_pConfig->SetCompareExtra(n, m_pChkCompareExtra[n]->GetValue());
		}
		m_pConfig->SetDownloadPlaylist(m_pDownloadPlaylist->GetValue());
		EndModal(wxID_OK);
		return;
	}
	if (event.GetId() == wxID_CANCEL)
	{
		event.Skip();
		return;
	}
	int nIndex = event.GetId()	- IDC_OPEN_DIR_DIALOG;
	wxASSERT(nIndex >= 0 && nIndex < m_nDirsUsed);
	wxString strInitialDir = m_pDir[nIndex]->GetValue();
	// always try to open in some default directory if possible
	if (strInitialDir.IsEmpty())
	{
		for (n = 0; n < m_nDirsUsed; n++)
		{
			if (!m_pDir[n]->GetValue().IsEmpty())
			{
				strInitialDir = m_pDir[n]->GetValue();
				break;
			}
		}
	}
#ifndef __WINDOWS__
	if (strInitialDir.IsEmpty()) strInitialDir = wxGetHomeDir();
#endif
	wxDirDialog dlg(this, _T("Select location"), strInitialDir);
	if (dlg.ShowModal() == wxID_OK)
	{
		m_pDir[nIndex]->SetValue(dlg.GetPath());
	}
}


