#include "Precompiled.h"
#include "EzTag.h"
#include <id3/misc_support.h>

const ID3_FrameID c_frameID[nMaxFrameTypes] = 
	{ID3FID_NOFRAME, ID3FID_TITLE, ID3FID_LEADARTIST, ID3FID_ALBUM, 
	ID3FID_CONTENTTYPE, ID3FID_NOFRAME, ID3FID_NOFRAME, ID3FID_NOFRAME, 
	ID3FID_TRACKNUM, ID3FID_YEAR};

EzTag::EzTag()
{
}

EzTag::~EzTag()
{
}

void EzTag::UpdateTag(int nFrameIDIndex, const wxChar *text)
{
	wxASSERT(nFrameIDIndex > 0 && nFrameIDIndex < nMaxFrameTypes);
	ID3_Frame* frame = NULL;
	if (NULL != text)
	{
		// first remove old tag
		do
		{
			frame = Find(c_frameID[nFrameIDIndex]);
			if (frame)
			{
				frame = RemoveFrame(frame);
				delete frame;
			}
		} while(frame);
		if (strlen(text) > 0)
		{
			frame = new ID3_Frame(c_frameID[nFrameIDIndex]);
			frame->GetField(ID3FN_TEXT)->Set(text);
			AttachFrame(frame);
		}
	}
}

wxString EzTag::GetField(ID3_FrameID fid)
{
	wxString str;
	ID3_Frame* frame = Find(fid); // find frame
	if (frame == NULL) return str;
	char *psz = NULL;
	ID3_Field* fld;
	fld = frame->GetField(ID3FN_TEXT);
	if (fld == NULL) return str;
	ID3_TextEnc enc = fld->GetEncoding();
	// compensate for bug in id3lib, force to convert to
	// unicode and back again (otherwise we can't read the data)
	if (enc == ID3TE_UTF8)
	{
		fld->SetEncoding(ID3TE_UNICODE);
	}
	fld->SetEncoding(ID3TE_ASCII);
	size_t nSizeText = fld->Size();
	if (nSizeText > 0)
	{
		psz = new char[nSizeText + 1];
		fld->Get(psz, nSizeText + 1);
		fld->SetEncoding(enc);
		str = psz;
		if (psz) delete[] psz;
		str.Trim(true);
		str.Trim(false);
	}
	return str;
}

