#include "Precompiled.h"
#include <libnjb.h>
#include "LNomad.h"
#include "Playlist.h"


const int c_nUnknownError = -2;

static njb_struct m_parrayJBs[NJB_MAX_DEVICES];

void songid_frame_read(njb_songid_frame_t* pFrame, wxString& strValue, int& nValue);

Nomad::Nomad() : m_pJukeBox(NULL), m_pSongID(NULL), m_bConnected(false), 
	m_nCurrentDone(0), m_bCancelTransfer(false) 
{
}

Nomad::~Nomad()
{
	if(m_pJukeBox != NULL)
	{
		if (m_pSongID != NULL) NJB_Songid_Destroy(m_pSongID);
		int nRet = NJB_Release(m_pJukeBox);
		if (nRet == -1)
		{
			wxLogMessage("Error in NJB_Release\n");			
			wxLogMessage(GetLastErrorStr());
		}
		NJB_Close(m_pJukeBox);
//		delete [] m_parrayJBs;
	}
}


bool Nomad::InitJukeBox()
{
	if (m_bConnected) return true;
	int nCountJBs = 0;
//	m_parrayJBs = new njb_t[NJB_MAX_DEVICES];
	int nRet = NJB_Discover(m_parrayJBs, 0, &nCountJBs);
	if (nRet == -1)
	{
		wxLogMessage(GetLastErrorStr());
		wxLogMessage("Error scanning Nomads\n");			
	}
	// note: return value of -1 doesn't necessarily mean no JB was found!
	if (nCountJBs == 0) return false;
	m_pJukeBox = m_parrayJBs;
	if (NJB_Open(m_pJukeBox) == -1 ) 
	{
		wxLogMessage(GetLastErrorStr());
		return false;
	}
	NJB_Capture(m_pJukeBox);
	// Test to see correct version of njblib
	if (false)
	{
		// If this line fails to compile then it means that you are using
		// the wrong version of libnjb (should be 1.2)!
		NJB_Reset_Get_EAX_Type(m_pJukeBox);
	}
	m_bConnected = true;
	return true;
}

Playlist* Nomad::GetFirstPlaylist()
{
	if (!m_bConnected) return false;
	NJB_Reset_Get_Playlist(m_pJukeBox);
	return GetNextPlaylist();
}	//	GetFirstPlaylist

Playlist* Nomad::GetNextPlaylist()
{
	njb_playlist_t *pLPlaylist;
	pLPlaylist = NJB_Get_Playlist(m_pJukeBox); 
	if (pLPlaylist == NULL) return NULL;
	Playlist* pPlaylist = new Playlist;
	pPlaylist->SetItemID((unsigned int)pLPlaylist->plid);
	pPlaylist->SetName(pLPlaylist->name);
	
	njb_playlist_track_t *track;
	map<unsigned int, Track*>::iterator iterMapTrack;
	NJB_Playlist_Reset_Gettrack(pLPlaylist);
	while ( (track = NJB_Playlist_Gettrack(pLPlaylist)) ) 
	{
		iterMapTrack = m_mapTrack.find(track->trackid); 
		if (iterMapTrack == m_mapTrack.end()) 
		{
			wxLogMessage("Unknown track in playlist: %s!", pLPlaylist->name);
		}
		else pPlaylist->Add(iterMapTrack->second);
	}
	NJB_Playlist_Destroy(pLPlaylist);
	return pPlaylist;
}	//	GetFirstPlaylist


bool Nomad::GetPlaylistTracks()
{
	return true;	// this function only makes sense in windows version
}

bool Nomad::SendPlaylist(Playlist* pPlaylist)
{
	njb_playlist_t *pLPlaylist;
	pLPlaylist = NJB_Playlist_New();
	NJB_Playlist_Set_Name(pLPlaylist, pPlaylist->GetName());
	if (pPlaylist->GetItemID() != 0)
	{
		pLPlaylist->plid = pPlaylist->GetItemID();
		pLPlaylist->_state = NJB_PL_CHTRACKS;
	}
	TrackList& listTrack = pPlaylist->GetNomadListTrack();
	unsigned int n;
	njb_playlist_track_t* pPLTrack;
	for (n = 0; n < listTrack.size(); n++)
	{
		pPLTrack = NJB_Playlist_Track_New(listTrack[n]->GetItemID());
		NJB_Playlist_Addtrack(pLPlaylist, pPLTrack, NJB_PL_END);
	}
	int nRet = NJB_Update_Playlist(m_pJukeBox, pLPlaylist);
	m_nLastError = nRet != 0;
	// if there's an error njb_error doesn't show any error code
//	if (nRet == -1 &&  m_nLastError == 0) m_nLastError = c_nUnknownError;
	// bug in njblib, doesn't return correct plid! (we have to add 3)
	pPlaylist->SetItemID((unsigned int)pLPlaylist->plid);
	NJB_Playlist_Destroy(pLPlaylist);
	return nRet == 0;
}	// SendPlaylist

bool Nomad::DeleteItem(const Playlist* pPlaylist)
{
	if (pPlaylist->GetItemID() == 0)
	{
		wxASSERT(false);
		return false;
	}
	int nRet = NJB_Delete_Playlist(m_pJukeBox, pPlaylist->GetItemID());
	m_nLastError = nRet != 0;
	// if there's an error njb_error doesn't show any error code
//	if (nRet == -1 &&  m_nLastError == 0) m_nLastError = c_nUnknownError;
	return nRet == 0;
}

bool Nomad::StartLoadAllTracks()
{
	if (!m_bConnected) return false;
	RemoveTracks();
	m_nCount = 0;
	NJB_Reset_Get_Track_Tag(m_pJukeBox);
	if (m_pSongID != NULL) 
	{
		NJB_Songid_Destroy(m_pSongID);
		m_pSongID = NULL;
	}
	m_pSongID = NJB_Get_Track_Tag(m_pJukeBox);
	return m_pSongID != NULL;
}

bool Nomad::MoveNextTrack()
{
	if (!m_bConnected || m_pSongID == NULL) return false;
	NJB_Songid_Destroy(m_pSongID);
	m_pSongID = NULL;
	m_nCount++;
	m_pSongID = NJB_Get_Track_Tag(m_pJukeBox);
	return m_pSongID != NULL;
}

Track* Nomad::LoadTrackData()
{
	if (!m_bConnected || m_pSongID == NULL) return false;
	Track* pTrack = new Track;
	pTrack->SetItemID(m_pSongID->trid);
	njb_songid_frame_t *pFrame;
	NJB_Songid_Reset_Getframe(m_pSongID);
	int nValue;
	wxString strValue;
	while ( (pFrame = NJB_Songid_Getframe(m_pSongID)) ) 
	{
		songid_frame_read(pFrame, strValue, nValue);
		// to do: test this!
		if (strncasecmp((const char*)pFrame->label, "PLAYONLY", 8) == 0
			&& nValue)
		{
			wxLogMessage("Protected track ignored: %s!", 
				(const char*)pTrack->GetDisplayName());
			AddTrack(pTrack);
			return NULL;
		}
		SetTrackAttrib(pTrack, (const char*)pFrame->label, strValue, nValue);
	}
	AddTrack(pTrack);
	return pTrack;
}	// GetCurrTrackData


void songid_frame_read(njb_songid_frame_t* pFrame, wxString& strValue, int& nValue)
{
	if (pFrame->type == NJB_TYPE_STRING) 
	{
		char *psz = strValue.GetWriteBuf(strlen(pFrame->data.strval) + 1);
		strcpy(psz, pFrame->data.strval);
		strValue.UngetWriteBuf();
	} 
	else 
	{
		if (pFrame->type == NJB_TYPE_UINT16) nValue = pFrame->data.u_int16_val;
		else if (pFrame->type == NJB_TYPE_UINT32) nValue = pFrame->data.u_int32_val;
		else
		{
			wxLogMessage("Unknown data type: %d", pFrame->type);			
		}
	}
}

bool Nomad::DeleteItem(const Track* pTrack)
{
	if (!m_bConnected) return false;
	int nRet = NJB_Delete_Track(m_pJukeBox, pTrack->GetItemID());
	m_nLastError = nRet != 0; 
	return nRet == 0;
}


bool Nomad::SendLabel(const Track* pTrackPC, 
		const Track* pTrackNomad)
{
	// It's always undesirable to upload blank data for Track num and year
	int nTrackNum = pTrackPC->GetTrackNum();
	if (nTrackNum == 0)
	{
		nTrackNum = pTrackNomad->GetTrackNum();
	}
	int nYear = pTrackPC->GetYear();
	if (nYear == 0)
	{
		nYear = pTrackNomad->GetYear();
	}
	
	njb_songid_t *songid;
	njb_songid_frame_t *frame;
	
	songid = NJB_Songid_New();
	frame = NJB_Songid_Frame_New_Codec(pTrackPC->GetAttrib(taCodec).Upper());
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Filesize(pTrackNomad->GetFileSize());
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Title(pTrackPC->GetFilledAttrib(taTitle));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Album(pTrackPC->GetFilledAttrib(taAlbum));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Artist(pTrackPC->GetFilledAttrib(taArtist));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Genre(pTrackPC->GetFilledAttrib(taGenre));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Year(nYear);
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Tracknum(nTrackNum);
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Length(pTrackPC->GetTrackLength());
	NJB_Songid_Addframe(songid, frame);
	
	int nRet = NJB_Replace_Track_Tag(m_pJukeBox, pTrackNomad->GetItemID(), songid);
	m_nLastError = nRet != 0; 
	NJB_Songid_Destroy(songid);
	return nRet == 0;
}

bool Nomad::GetItem(const Track* pTrack, wxFileName& fileSpec)
{
	m_bCancelTransfer = false;
	int nRet = NJB_Get_Track(m_pJukeBox, pTrack->GetItemID(), pTrack->GetFileSize(), 
		fileSpec.GetFullPath(), &Nomad::WrapperProgressUp, this);
	if (m_bCancelTransfer) return true;
	m_nLastError = nRet != 0; 
	return nRet == 0;
}

bool Nomad::SendItem(Track* pTrack)
{
	u_int32_t trackid;
	m_bCancelTransfer = false;
	
	
	njb_songid_t *songid;
	njb_songid_frame_t *frame;
	
	songid = NJB_Songid_New();
	frame = NJB_Songid_Frame_New_Codec(pTrack->GetAttrib(taCodec).Upper());
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Title(pTrack->GetFilledAttrib(taTitle));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Album(pTrack->GetFilledAttrib(taAlbum));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Artist(pTrack->GetFilledAttrib(taArtist));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Genre(pTrack->GetFilledAttrib(taGenre));
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Year(pTrack->GetYear());
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Tracknum(pTrack->GetTrackNum());
	NJB_Songid_Addframe(songid, frame);
	frame = NJB_Songid_Frame_New_Length(pTrack->GetTrackLength());
	NJB_Songid_Addframe(songid, frame);
	
	int nRet = NJB_Send_Track(m_pJukeBox, pTrack->GetFilledAttrib(taFileSpec), 
		songid, &Nomad::WrapperProgressDown, this, &trackid);
	if (m_bCancelTransfer) return true;
	if (nRet == 0) pTrack->SetItemID(trackid);
	m_nLastError = nRet != 0; 
	return nRet == 0;
}

int Nomad::ProgressDown(u_int64_t nSent, u_int64_t total, const char* buf, unsigned len)
{
	m_nCurrentDone = nSent;
	if (m_bCancelTransfer) 
	{
		wxLogMessage("Download canceled Progress %d/%d", (int)nSent, (int)total);			
		return -1;
	}
	return 0;
}
		 
int Nomad::WrapperProgressDown(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data)
{
	Nomad* pNomad = (Nomad*)data;
	if (pNomad)	
	{
		return pNomad->ProgressDown(sent, total, buf, len);
	}
	return 0;
}

int Nomad::ProgressUp(u_int64_t nSent, u_int64_t total, const char* buf, unsigned len)
{
	m_nCurrentDone = nSent;
	if (m_bCancelTransfer) 
	{
		wxLogMessage("Upload canceled Progress %d/%d", (int)nSent, (int)total);			
		return -1;
	}
	return 0;
}
		 
int Nomad::WrapperProgressUp(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data)
{
	Nomad* pNomad = (Nomad*)data;
	if (pNomad)	
	{
		return pNomad->ProgressUp(sent, total, buf, len);
	}
	return 0;
}

wxString Nomad::GetLastErrorStr() const
{
	if (m_nLastError == c_nUnknownError) 
	{
		m_strLastError = "Unknown error";
		return m_strLastError;
	}
	NJB_Error_Reset_Geterror(m_pJukeBox);
	const char* szErr;
	m_strLastError.Empty();
	while ( (szErr = NJB_Error_Geterror(m_pJukeBox)) != NULL ) 
	{
		m_strLastError += szErr;
	}
	return m_strLastError;
}
