#include "Precompiled.h"
#include "LTransferDlg.h"
#include "LNomad.h"
#include "NomadSyncDlg.h"
#include "Track.h"
#include "Action.h"
#include "EzMessageBox.h"

BEGIN_EVENT_TABLE(TransferDlg, BaseTransferDlg)
	EVT_BUTTON(ID_ABORT, BaseTransferDlg::OnAbort)
	EVT_TIMER(-1, TransferDlg::OnTimer)
END_EVENT_TABLE()


TransferDlg::TransferDlg(ActionTable* pActionTable, Nomad* pNomad)
	: BaseTransferDlg(pActionTable, pNomad), 
	m_pThreadWorker(NULL), m_bThreadFinished(false)
{
}

TransferDlg::~TransferDlg() 
{
}


//#define TEST_WITHOUT_THREAD 1

bool TransferDlg::TransferDataToWindow()
{
	Init();
#ifndef TEST_WITHOUT_THREAD 
	m_timer.SetOwner(this);
	m_dBytesDone = 0.0;
	StartThread();
#else
	int nBytesDone = 0;
	for (unsigned int n = 0; n < m_listAction.size(); n++)
	{
		if (m_listAction[n]->IsPending())
		{
			m_plblCurrTrack->SetLabel(
				m_listAction[n]->GetDisplayStr());
			m_listAction[n]->DoAction(m_pNomad);
			m_listAction[n]->DoPostAction(0);
			nBytesDone += m_listAction[n]->GetTimeNeeded();
			m_pProgress->SetValue(nBytesDone*100/m_dBytesTotal);
			m_pActionTable->ForceRefresh();
			m_pNomad->SetCurrentDone(0);
		}
	} 
#endif
	return wxDialog::TransferDataToWindow();
}

void TransferDlg::StartThread()
{
	m_listActionDone.clear();
	m_bThreadFinished = false;
	m_pThreadWorker = new WorkerThread(this, m_listAction, m_pNomad);
	m_pThreadWorker->Create();
	m_pThreadWorker->Run();
	m_timer.Start(200);
}

// if there are no action items to do
// m_listActionsDone will be empty and the
// worker thread will be finished
void TransferDlg::OnTimer(wxTimerEvent &WXUNUSED(event) )
{
	if (m_bProcessing) return;
	m_bProcessing = true;
	wxLogMessage("OnTimer: waiting for mutex lock");			
	m_mutex.Lock();
	wxLogMessage("OnTimer: got lock");			
	Action* pCurrAction = NULL;
	if (m_listActionDone.size() > 0)
	{
		pCurrAction = m_listActionDone[m_listActionDone.size()-1];
		if (m_listActionDone.size() > 1)
		{
			wxLogMessage("more than one action completed was pending display");			
			m_pActionTable->ForceRefresh();
			m_listActionDone.clear();
			m_listActionDone.push_back(pCurrAction);
		}
	}
	m_plblCurrTrack->SetLabel(m_strCurrentAction);
	m_mutex.Unlock();
	wxLogMessage("OnTimer: mutex unlocked");			
	m_pActionTable->ScrollToAction(pCurrAction);
	if (m_bThreadFinished) 
	{
		wxLogMessage("thread finished");			
		m_timer.Stop();
		m_pActionTable->ForceRefresh();
		if (pCurrAction && pCurrAction->HadError())
		{
			int nRet;
			nRet = EzMessageBox("Error", pCurrAction->GenErrorMsg(), 
				0, "Continue", "Abort");
			if (nRet == 1) 
			{
				StartThread();
				m_bProcessing = false;
				return;
			}
		}	
		wxLogMessage("kill transfer dialog");			
		Close();
		m_bProcessing = false;
		return;
	}
	SetProgress(m_dBytesDone + m_pNomad->GetCurrentDone());
	m_bProcessing = false;
}

void TransferDlg::SetCurrentAction(Action* pAction)
{
	wxLogMessage("SetCurrentAction: waiting for mutex lock");			
	wxMutexLocker lock(m_mutex);
	wxLogMessage("SetCurrentAction: got lock");			
	m_listActionDone.push_back(pAction);
	// keeps a copy of this so that there is no concurrent acces to the curr Action object
	m_strCurrentAction = pAction->GetDisplayStr();
	wxLogMessage("SetCurrentAction: %s", (const char*)m_strCurrentAction);
}

//----------------------------------------------------------------

WorkerThread::WorkerThread(TransferDlg* pParent, ActionList &listAction, Nomad* pNomad)
	: m_listAction(listAction), m_pNomad(pNomad), m_pParent(pParent)
{
}

void WorkerThread::OnExit()
{
	wxLogMessage("WorkerThread::OnExit");			
	m_pParent->SetThreadFinished(true);
}


wxThread::ExitCode WorkerThread::Entry()
{
	wxLogMessage("Worker thread Entry");			
	m_pNomad->SetCurrentDone(0);
	for (unsigned int n = 0; n < m_listAction.size(); n++)
	{
		if (m_listAction[n]->IsPending())
		{
			m_pParent->SetCurrentAction(m_listAction[n]); 
			m_listAction[n]->DoAction(m_pNomad);
			if (m_pNomad->GetCancelTransfer()) 
			{
				m_listAction[n]->SetExecuting(false);
				break;
			}
			wxLogMessage("Action done");			
			m_listAction[n]->DoPostAction(0);
			m_pParent->AddBytesDone(m_listAction[n]->GetTimeNeeded());
			if (m_pParent->GetAbort() || m_listAction[n]->HadError()) 
			{
				wxLogMessage("aborted or had error");
				wxLogMessage(m_listAction[n]->GenErrorMsg());
				break;
			}
			m_pNomad->SetCurrentDone(0);
		}
	} 
	wxLogMessage("exit WorkerThread::Entry");			
	return 0;
}





