#include "Precompiled.h"
#include "NomadSyncDlg.h"
#include "EzGrid.h"
#include "ConfigDlg.h"
#include <wx/config.h>
#include "TrackMatcher.h"
#include "Track.h"	
#include "Action.h"
#include "EzMessageBox.h"
#include <wx/mimetype.h>
#include <wx/clipbrd.h>
#include "PlaylistMatcher.h"
#include "CommonFunc.h"
#include "TrackCache.h"


#ifdef __WINDOWS__
	#include "WNomad.h"
	#include "WTransferDlg.h"
#else
	#include "LTransferDlg.h"
	#include "LNomad.h"
	#include "camelHead.xpm"
#endif
#include "search.xpm"

const int ID_COPY = 101;
const int ID_SHOW_PATH = 102;
const int ID_PLAY = 103;
const int IDC_UPDATE = 1001;
const int IDC_SEARCH = 1002;
const int IDC_CONFIG = 1003;
const int IDC_EXIT = 1004;
const int IDC_ABOUT = 1005;
const int IDC_HTMLHELP = 1006;
const int IDC_PLAYLISTS = 1007;


BEGIN_EVENT_TABLE(NomadSyncDlg, wxDialog)
	EVT_CLOSE(NomadSyncDlg::OnQuit)
	EVT_BUTTON(IDC_UPDATE, NomadSyncDlg::OnUpdate)
	EVT_BUTTON(IDC_SEARCH, NomadSyncDlg::OnSearch)
	EVT_BUTTON(IDC_PLAYLISTS, NomadSyncDlg::OnPlaylists)
	EVT_BUTTON(IDC_CONFIG, NomadSyncDlg::OnConfig)
	EVT_BUTTON(IDC_HTMLHELP, NomadSyncDlg::OnHTMLHelp)
	EVT_BUTTON(IDC_EXIT, NomadSyncDlg::OnQuit)
	EVT_BUTTON(IDC_ABOUT, NomadSyncDlg::OnAbout)
	EVT_GRID_CELL_CHANGE(NomadSyncDlg::OnCellValueChanged)
	EVT_MENU(-1, NomadSyncDlg::OnContextMenu)
	EVT_GRID_LABEL_LEFT_CLICK(NomadSyncDlg::OnLabelLeftClick)
	EVT_GRID_CELL_RIGHT_CLICK(NomadSyncDlg::OnRightClick)
	EVT_GRID_LABEL_RIGHT_CLICK(NomadSyncDlg::OnRightClick)
END_EVENT_TABLE()


NomadSyncDlg::NomadSyncDlg(const wxString& title) : 
	wxDialog((wxDialog *)NULL, -1, title, wxDefaultPosition, wxDefaultSize, 
		wxSYSTEM_MENU | wxRESIZE_BORDER | wxMINIMIZE_BOX | wxMAXIMIZE_BOX | 
		wxTHICK_FRAME | wxSYSTEM_MENU | wxCAPTION), 
		m_pDlgTransfer(NULL), m_pTrackMatcher(NULL), m_pPlaylistMatcher(NULL),
		m_pLogFile(NULL), m_nRightClickRow(-1)
{
#ifdef WXDEBUG
   delete wxLog::SetActiveTarget(new wxLogStderr(NULL));
#else
	wxLog::EnableLogging(FALSE);
#endif
	m_pNomad = new Nomad();
	m_pDlgTransfer = new TransferDlg(&m_tableAction, m_pNomad);  
	Init();
	m_pTrackCache = new TrackCache;
	m_pTrackCache->ReadCacheFile(m_config.GetUploadDir() + "cached_files.txt");
	GetMp3OpenCommand();
}

NomadSyncDlg::~NomadSyncDlg()
{
	m_pTrackCache->WriteCacheFile(m_config.GetUploadDir() + "cached_files.txt");
	wxConfigBase *cfg = wxConfigBase::Get();
	if (IsMaximized()) cfg->Write("maximized", 1);
	else
	{
		cfg->Write("maximized", 0);
		cfg->Write(_T("x"), (long)GetPosition().x);
		cfg->Write(_T("y"), (long)GetPosition().y);
		cfg->Write(_T("width"), (long)GetSize().x);
		cfg->Write(_T("height"), (long)GetSize().y);
	}
	delete m_pNomad;
	delete m_pDlgTransfer;
	delete wxLog::SetActiveTarget(NULL);
	if (m_pLogFile != NULL)
	{
		fclose(m_pLogFile);
	}
	if (m_pTrackMatcher) delete m_pTrackMatcher;
	if (m_pPlaylistMatcher) delete m_pPlaylistMatcher;
	delete m_pTrackCache;
}	// ~NomadSyncDlg


void NomadSyncDlg::Init()
{
	#ifdef __WINDOWS__
		wxIcon icon("IDI_CAMEL_HEAD");
	#else
		wxIcon icon(szIconCamelHead);
	#endif
	SetIcon(icon);		// set the main program icon
	wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );	// controls placement of controls
	m_pGrid = new EzGrid( this, -1, wxPoint( 10, 10 ), wxSize( 480, 350 ) );
	topsizer->Add(m_pGrid,
		1,            // make vertically stretchable
		wxADJUST_MINSIZE| wxGROW |    // make horizontally stretchable
		wxALL,        //   and make border all around
		3 );         // border width 
	m_tableAction.SetGrid(m_pGrid);
	// FALSE means don't delete table!
	m_pGrid->SetTable(&m_tableAction, FALSE, wxGrid::wxGridSelectRows); 
	// set default column sizes
	m_pGrid->SetColSize(c_nIconCol, 20);
	m_pGrid->SetColSize(c_nActionCol, 100);
	m_pGrid->SetColSize(c_nTitleCol, 120);
	m_pGrid->SetColSize(c_nArtistCol, 120);
	m_pGrid->SetColSize(c_nAlbumCol, 120);
	m_pGrid->SetColSize(c_nGenreCol, 80);
	m_pGrid->SetColSize(c_nFileSizeCol, 60);
	m_pGrid->SetColSize(c_nLengthCol, 50);
	m_pGrid->SetColSize(c_nTrackNumCol, 50);
	m_pGrid->SetColSize(c_nYearCol, 50);
	m_pGrid->SetColSize(c_nFileSpecCol, 200);
	m_pGrid->PreviousState();	// load size and postion from registry
	m_pGrid->EnableDragRowSize(false);
	m_pGrid->SetMargins(0, 0);
	m_pGrid->RecalcRowLabel();
	m_pGrid->SetDefaultCellBackgroundColour(
		wxSystemSettings::GetSystemColour(wxSYS_COLOUR_BTNFACE));
	wxButton* config = new wxButton(this, IDC_CONFIG, wxT("&Configure..."), wxPoint(-1, -1),
					   wxSize( 80, 29) );
	wxIcon iconSearch(szSearch);
	m_pBtnSearch = new wxIconButton(this, IDC_SEARCH, iconSearch, config->GetFont(), 
						wxPoint(-1, -1), wxSize( 80, 29), wxBU_AUTODRAW, 
						wxDefaultValidator, "Tracks");
	m_pBtnPlaylists = new wxIconButton(this, IDC_PLAYLISTS, iconSearch, config->GetFont(), 
						wxPoint(-1, -1), wxSize( 80, 29), wxBU_AUTODRAW, 
						wxDefaultValidator, "Playlists");
	if (!m_config.IsIncludeDirSet()) 
	{
		m_pBtnSearch->Enable(false);
		m_pBtnPlaylists->Enable(false);
	}
	m_pBtnUpdate = new wxButton(this, IDC_UPDATE, wxT("&Do Actions"), wxPoint(-1, -1),
					   wxSize( 100, 29) );
	m_pBtnUpdate->Enable(false);
//	wxButton* help = new wxButton(this, IDC_HTMLHELP, wxT("&Help..."), wxPoint(-1, -1),
//					   wxSize( 80, 29) );
	wxButton* exit = new wxButton(this, IDC_EXIT, wxT("E&xit"), wxPoint(-1, -1),
					   wxSize( 80, 29) );
	wxButton* about = new wxButton(this, IDC_ABOUT, wxT("&About..."), wxPoint(-1, -1),
					   wxSize( 80, 29) );
	m_pLblStatus = new wxStaticText(this, -1, "", wxPoint(-1, -1),
					   wxSize(200, 29), wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

	wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
	button_sizer->Add(m_pBtnSearch, 0, wxALL, 3);
	m_pBtnSearch->SetToolTip("Search and compare tracks");
	button_sizer->Add(m_pBtnPlaylists, 0, wxALL, 3);
	m_pBtnPlaylists->SetToolTip("Search and compare playlists");
	button_sizer->Add(m_pBtnUpdate, 
		0,           // make horizontally unstretchable
		wxALL,       // make border all around (implicit top alignment)
		3);        // set border width to 3
	button_sizer->Add(config, 0, wxALL, 3);        
//	button_sizer->Add(help, 0, wxALL, 3);        
	button_sizer->Add(exit, 0, wxALL, 3);        
	button_sizer->Add(about, 0, wxALL, 3);        
	button_sizer->Add(m_pLblStatus, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);        
	topsizer->Add(button_sizer, 0, wxALIGN_LEFT); // no border 
	SetSizer(topsizer);      // use the sizer for layout
	topsizer->SetSizeHints(this);   // set size hints to honour minimum size
	wxConfigBase *cfg = wxConfigBase::Get();
	// Set previously stored size
	wxRect rect(wxPoint(cfg->Read("x", -1), cfg->Read("y", -1)),
		wxSize(cfg->Read("width", -1), cfg->Read("height", -1)));
	SetSize(rect);
	if (cfg->Read("maximized", (long)0) == 1)	Maximize();
}	// Init


void NomadSyncDlg::OnQuit(wxCommandEvent& WXUNUSED(event))
{
   Destroy();
}

void NomadSyncDlg::OnConfig(wxCommandEvent& WXUNUSED(event))
{
	ConfigDlg dlg(this, &m_config, m_pNomad);
	if (dlg.ShowModal() == wxID_OK)
	{
		if (!m_config.GetLinuxMP3Command().IsEmpty()) GetMp3OpenCommand();
		if (m_config.IsIncludeDirSet()) 
		{
			m_pBtnSearch->Enable(true);
			m_pBtnPlaylists->Enable(true);
		}
		else
		{
			m_pBtnSearch->Enable(false);
			m_pBtnPlaylists->Enable(false);
		}
		if (m_config.IsChangedDefault())
		{
			wxBusyCursor wait;
			m_tableAction.SetDefaultActions();
			CheckActionsToDo();
		}
		wxLog::EnableLogging(m_config.GetEnableLog());
		if (m_config.GetEnableLog())
		{
			if (m_pLogFile == NULL)
			{
				m_pLogFile = fopen(m_config.GetUploadDir()+"log.txt", "w+");
				delete wxLog::SetActiveTarget(new wxLogStderr(m_pLogFile));
			}
			int n;
			for (n = c_nFirstIncludeDir; n <= c_nLastExcludeDir; n++)
			{
				wxLogMessage(m_config.GetDir(n));
			}
		}
	}
}		// OnConfig


void NomadSyncDlg::OnHTMLHelp(wxCommandEvent& WXUNUSED(event))
{
	wxFileName fn;
	fn.AssignDir(wxGetCwd());
	fn.SetFullName("NomadSyncHelp.html");
//	ViewHTMLFile(wxNativePathToURL(fn.GetFullPath()));
}

void NomadSyncDlg::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	wxString str;
	str = "Synchronisation utility for Creative Jukebox audio players\n";
	str += "Version 0.4.2\n";
	str += "http://nomadsync.sourceforge.net\n";
	str += "\n";
	str += "This program uses:\n";
	str += "id3lib    http://id3lib.sourceforge.net/\n";
	str += "  wxWidgets   http://www.wxwidgets.org/\n";
#ifndef __WINDOWS__
	str += "  libnjb   http://libnjb.sourceforge.net/\n";
#endif
	str += "\n";
	str += "Copyright (C) 2004 Ed Welch";
	EzMessageBox("About NomadSync", str, 0, "OK");
}

// Searches list of mp3s on Nomad & PC and displays descrepencies
void NomadSyncDlg::OnSearch(wxCommandEvent& WXUNUSED(event))
{
	m_tableAction.SetMode(modeTrack);
	if (m_pTrackMatcher == NULL) 
		m_pTrackMatcher = new TrackMatcher(m_pTrackCache, m_config);
	Search(m_pTrackMatcher, "Searching for MP3s...");
} // OnStart


void NomadSyncDlg::OnPlaylists(wxCommandEvent& WXUNUSED(event))
{
	m_tableAction.SetMode(modePlaylist);
	if (m_pPlaylistMatcher == NULL) 
		m_pPlaylistMatcher = new PlaylistMatcher(m_pTrackCache, m_config);
	Search(m_pPlaylistMatcher, "Searching for Playlists...");
} 


void NomadSyncDlg::Search(Matcher* pMatcher, const char* szMessage)
{
	m_pLblStatus->SetLabel("Opening Jukebox...");
	wxSafeYield();		// wxSafeYield allows the UI to update
	// wxSafeYield interferes with the cursor, so we have to manually set it
	wxBeginBusyCursor();	
	if (!m_pNomad->InitJukeBox())
	{
		wxEndBusyCursor();
		m_pLblStatus->SetLabel("");
		wxMessageBox(wxT("Could not initialise Jukebox!"));
		return;
	}
	wxEndBusyCursor();
	m_pLblStatus->SetLabel(szMessage);
	wxSafeYield();
	wxBeginBusyCursor();
	// we must disable the edit control before we recreate the grid
	if (m_pGrid->IsCellEditControlShown())	
	{
		m_pGrid->EnableCellEditControl(false);
	}
//	m_tableAction.RemoveActions();
//	m_pGrid->UpdateNumRows();	// call update to change the number of rows
	pMatcher->Run(m_pNomad);
	pMatcher->CreateActions(m_tableAction);
	m_pLblStatus->SetLabel("");
	m_pGrid->UpdateNumRows();	// call update to change the number of rows
	m_pGrid->RecalcRowLabel();
	m_pGrid->ForceRefresh();
	m_pGrid->SetFocus();
	wxEndBusyCursor();
	CheckActionsToDo();
} 

// OnUpdate called when user want to carry out the actions
void NomadSyncDlg::OnUpdate(wxCommandEvent& WXUNUSED(event))
{
	wxBusyCursor wait;
	wxString strAction;
	if (m_pGrid->IsCellEditControlShown())	
	{
		m_pGrid->DisableCellEditControl();		
	}
	m_tableAction.GetActionsInOrder(m_pDlgTransfer->GetActionList());
	if (m_pDlgTransfer->GetActionList().size() > 0)
	{
		m_pDlgTransfer->ShowModal();
		// add downloaded files to the playlist, if requested
		if (m_config.NeedPlaylists())
		{
			m_pNomad->AddTracksToPlaylist(m_config.GetDownloadPlaylist(), 
				m_pDlgTransfer->GetActionList()); 
			if (m_pNomad->HadError())
			{
				wxLogMessage("Error writing playlist: %s!", (const char*)m_pNomad->GetLastErrorStr());
				EzMessageBox("Error writing tracks to playlist", m_pNomad->GetLastErrorStr(), 
					0, "OK");
			}
		}
	}
	CheckActionsToDo();
}	//	OnUpdate


void NomadSyncDlg::OnCellValueChanged(wxGridEvent& ev)
{
	CheckActionsToDo();
	ev.Skip();
}	// OnCellValueChanged

void NomadSyncDlg::CheckActionsToDo()
{
	if (m_tableAction.ActionsToDo() > 0)
		m_pBtnUpdate->Enable(true);
	else
		m_pBtnUpdate->Enable(false);
}

void NomadSyncDlg::OnRightClick(wxGridEvent& event)
{
	wxMenu menu;
	menu.Append(ID_COPY, "Copy");
	if (!m_pGrid->GetCellValue(event.GetRow(), c_nFileSpecCol).IsEmpty())
	{
		menu.Append(ID_SHOW_PATH, "Show full file path");
		for (size_t n = 0; n < m_listMp3Commands.GetCount(); n++ )
		{
			menu.Append(ID_PLAY+n, m_listMp3Verbs[n]);
		}
	}
	m_nRightClickRow = event.GetRow();
	PopupMenu(&menu, event.GetPosition() );
}

// GetMp3OpenCommand reads the commands for the default mp3 player from registry
bool NomadSyncDlg::GetMp3OpenCommand()
{
	m_listMp3Verbs.Empty();
	m_listMp3Commands.Empty();
	if (!m_config.GetLinuxMP3Command().IsEmpty())
	{
		m_listMp3Verbs.Add("Play");
		m_listMp3Commands.Add(m_config.GetLinuxMP3Command() + " \"%s\"");
		return true;
	}
	wxString strCmd;
	wxFileType *ft = wxTheMimeTypesManager->GetFileTypeFromExtension("mp3");
	int nCount = 0;
	if (ft)
	{
		nCount = ft->GetAllCommands(&m_listMp3Verbs, &m_listMp3Commands, 
			wxFileType::MessageParameters("%s", ""));
		int nPlay = m_listMp3Verbs.Index("Play", FALSE);
		int nOpen = m_listMp3Verbs.Index("open", FALSE);
		// if both open and play commands exist, then remove play
		if (nOpen != wxNOT_FOUND && nPlay != wxNOT_FOUND)
		{
			m_listMp3Verbs.Remove(nOpen);
			m_listMp3Commands.Remove(nOpen);
		}
		delete ft;
	}
	return nCount > 0;
}

void NomadSyncDlg::OnContextMenu(wxCommandEvent& event)
{
  	if (m_nRightClickRow < 0) return;
	if (event.GetId() == ID_COPY)
	{
		if (wxTheClipboard->Open())
		{
			wxString str;
			for (int nCol = 0; nCol < m_pGrid->GetNumberCols(); nCol++)
			{
				str += m_pGrid->GetCellValue(m_nRightClickRow, nCol) + '\t';
			}
			str.RemoveLast();
			wxTheClipboard->SetData(new wxTextDataObject(str));
			wxTheClipboard->Close();
		}
		return;
	}
	if (event.GetId() == ID_SHOW_PATH)
	{
		int nWidth, nHeight;
		GetTextExtent(m_pGrid->GetCellValue(m_nRightClickRow, c_nFileSpecCol), 
			&nWidth, &nHeight);
		m_pGrid->SetColSize(c_nFileSpecCol, nWidth+4);
		m_pGrid->MakeCellVisible(m_nRightClickRow, c_nFileSpecCol);
		m_pGrid->ForceRefresh();
		return;
	}
	wxString strFile, strCmd;
	strFile = m_pGrid->GetCellValue(m_nRightClickRow, c_nFileSpecCol);
	unsigned int nIndex = event.GetId() - ID_PLAY;
	wxASSERT(nIndex >= 0 && nIndex < m_listMp3Verbs.GetCount());
	strCmd = wxString::Format(m_listMp3Commands[nIndex], (const char*)strFile);
	wxExecute(strCmd, FALSE);
}

void NomadSyncDlg::OnLabelLeftClick(wxGridEvent& event)
{
	if (event.GetCol() != -1)	// don't do anything if user clicked a row label
	{
		wxBusyCursor wait;
		m_tableAction.Sort(event.GetCol());
	}
	event.Skip();
} 
