#include "Precompiled.h"
#include "Playlist.h"
#include "Track.h"
#include "CommonFunc.h"


Playlist::Playlist(const Playlist& rhs)
{
	m_strName = rhs.m_strName;
	m_strFileSpec = rhs.m_strFileSpec;
	m_nItemID = rhs.m_nItemID;		
	m_nErrorMask = rhs.m_nErrorMask;
	unsigned int n;
	for (n = 0; n < rhs.m_listTrack.size(); n++)
	{
		m_listTrack.push_back(rhs.m_listTrack[n]);
	}
	for (n = 0; n < rhs.m_listTransTrack.size(); n++)
	{
		m_listTransTrack.push_back(rhs.m_listTransTrack[n]);
	}
}

bool Playlist::HasSameTracks(const Playlist* pPlayList) const
{
	if (m_listTrack.size() != pPlayList->m_listTrack.size()) return false;
	for (unsigned int n = 0; n < m_listTrack.size(); n++)
	{
		if (m_listTrack[n]->GetKey() != pPlayList->m_listTrack[n]->GetKey())
		{
			return false;
		}
	}
	return true;
}

void Playlist::Add(Track* pTrack)
{
	wxASSERT(pTrack->GetItemID() != 0);
	m_listTrack.push_back(pTrack);
}

void Playlist::AddNoDups(Track* pTrack)
{
	wxASSERT(pTrack->GetItemID() != 0);
	for (unsigned int n = 0; n < m_listTrack.size(); n++)
	{
		if (m_listTrack[n]->GetItemID() == pTrack->GetItemID())
		{
			wxLogMessage("Duplicate track not added to playlist"); 
			return;
		}
	}
	m_listTrack.push_back(pTrack);
}

bool Playlist::ParseFile(const TrackMap& mapPC)
{
	wxFile fileIn;
	if (!fileIn.Open(m_strFileSpec, wxFile::read))
	{
		wxLogMessage("Can't open playlist file %s", (const char*)m_strFileSpec);
		wxLogMessage(wxSysErrorMsg(wxSysErrorCode()));
		return false;
	}
	char* pBuff = new char[fileIn.Length()];
	fileIn.Read((void*)pBuff, fileIn.Length());
	const char*	pCurr = pBuff;
	const char* pStart;
	const char*	pEnd = pBuff+fileIn.Length();
	int nPos = m_strFileSpec.Find(wxFILE_SEP_PATH, true); // reverse find
	wxString strDir(m_strFileSpec.Left(nPos+1));
	wxString strTemp;
	wxString strNotPathSep, strPathSep;
	strPathSep = wxFILE_SEP_PATH;
	if (wxFILE_SEP_PATH == '/') strNotPathSep = "\\";
	else strNotPathSep = "/";

	TrackMap::const_iterator iterMap;
	do
	{
		pStart = pCurr;
		while (*pCurr != '\n' && *pCurr != '\r' && pCurr != pEnd) 
		{
			pCurr++;
		}
		wxString str(pStart, pCurr-pStart);
		if (!str.StartsWith("#EXT"))	// ignore extended m3u crap
		{
			// check relative filespec first
			str.Replace(strNotPathSep, strPathSep);
			strTemp = strDir+str;
			iterMap = mapPC.find(strTemp); 
			if (iterMap == mapPC.end()) 
			{
				iterMap = mapPC.find(str); 
			}
			if (iterMap != mapPC.end()) 
				m_listTrack.push_back(iterMap->second);
			else 
			{
				wxLogMessage("Track:%s in Playlist %s missing on PC", (const char*)str,
					(const char*)m_strName);
				m_nErrorMask |= c_nPCTracksMissing;
			}
		}
		while ((*pCurr == '\n' || *pCurr == '\r') && pCurr != pEnd) 
		{
			pCurr++;
		}
	} while (pCurr != pEnd);
	fileIn.Close();
	delete [] pBuff;
	return true;
}

bool Playlist::WriteM3uFile(const wxString& strDir)
{
	wxFileName fileSpec;
	fileSpec.AssignDir(strDir);
	fileSpec.SetName(m_strName);
	fileSpec.SetExt("m3u");
	wxASSERT(fileSpec.IsOk());
	wxFile fileOut;
	if (!fileOut.Open(fileSpec.GetFullPath(), wxFile::write))
	{
		wxLogMessage("Can't create playlist file %s", (const char*)fileSpec.GetFullPath());
		wxLogMessage(wxSysErrorMsg(wxSysErrorCode()));
		return false;
	}
	wxString str, strFile;
	for (unsigned int n = 0; n < m_listTransTrack.size(); n++)
	{
		wxASSERT(!m_listTransTrack[n]->IsNomad());
		strFile = m_listTransTrack[n]->GetAttrib(taFileSpec);
		strFile.StartsWith(strDir, &strFile);
		str += strFile + '\n';
	}
	fileOut.Write(str);
	fileOut.Close();
	return true;
}

void Playlist::TransformTracks(const TrackMap& mapTrack)
{
	wxString strKey;
	TrackMap::const_iterator iterMap;
	for (unsigned int n = 0; n < m_listTrack.size(); n++)
	{
		strKey = m_listTrack[n]->GetKey();
		iterMap = mapTrack.find(strKey); 
		if (iterMap != mapTrack.end()) 
		{
			m_listTransTrack.push_back(iterMap->second);
		}
		else
		{ 
			wxLogMessage("Transfer Track:%s in Playlist %s missing", (const char*)strKey, 
				(const char*)m_strName);
			if (m_listTrack[n]->IsNomad())
				m_nErrorMask |= c_nPCTracksMissing;
			else
				m_nErrorMask |= c_nNomadTracksMissing;
		}
	}
}

wxString Playlist::GetValue(int nCol) const 
{
	if (nCol == c_nIconCol) 
	{
		if (IsNomad())
			return "IDI_NOMAD";
		else
			return "IDI_PC";
	}
	if (nCol == c_nTitleCol) return m_strName;
	if (nCol == c_nTitleCol+1) 
	{
		return wxString::Format("%d", m_listTrack.size());
	}
	if (nCol == c_nTitleCol+2) 
	{
		if (!CanTransfer()) return "No tracks found!";
		if (m_nErrorMask == c_nPCTracksMissing) return "Tracks missing on PC";
		if (m_nErrorMask == c_nNomadTracksMissing) return "Tracks missing on Jukebox";
		if (m_nErrorMask == (c_nNomadTracksMissing | c_nPCTracksMissing)) 
			return "Tracks missing on PC and Jukebox";
	}
	if (nCol == c_nFileSpecCol) 
	{
		return m_strFileSpec;
	}
	wxString str;
	return str;
}

bool Playlist::ColIsLessThan(int nCol, const Playlist* pPlaylist) const
{
	if (nCol == c_nTitleCol) return m_strName < pPlaylist->m_strName;
	if (nCol == c_nFileSpecCol) 
	{
		return m_strFileSpec < pPlaylist->m_strFileSpec;
	}
	if (nCol == c_nTitleCol+1) 
	{
		return m_listTrack.size() < pPlaylist->m_listTrack.size();
	}
	if (nCol == c_nTitleCol+2) 
	{
		return GetValue(nCol) < pPlaylist->GetValue(nCol);
	}
	return false;
}
