#include "Precompiled.h"
#include "PlaylistMatcher.h"
#include "Config.h"
#include "Track.h"
#ifdef __WINDOWS__
	#include "WNomad.h"
#else
	#include "LNomad.h"
#endif
#include "ActionTable.h"
#include "Playlist.h"
#include "PlaylistAction.h"
#include "TrackCache.h"

PlaylistMatcher::~PlaylistMatcher()
{
	RemoveAll();
}

void PlaylistMatcher::RemoveAll()
{
	Matcher::RemoveAll();
	PlaylistMap::iterator iterMap; 
	for (iterMap = m_mapPCPlaylist.begin(); 
		iterMap != m_mapPCPlaylist.end(); iterMap++) 
		delete iterMap->second;
	for (iterMap = m_mapNomadPlaylist.begin(); 
		iterMap != m_mapNomadPlaylist.end(); iterMap++) 
		delete iterMap->second;
	m_mapPCPlaylist.clear();
	m_mapNomadPlaylist.clear();
	m_listPCExtraPlaylist.clear();
	unsigned int n;
	for (n = 0; n < m_listDiffPlaylist.size(); n++)
		delete m_listDiffPlaylist[n].m_pNomad;
	m_listDiffPlaylist.clear();
}

void PlaylistMatcher::Run(Nomad* pNomad)
{
	RemoveAll();
	m_pTrackCache->RemoveHits();
	TraverseDirs("*.mp3");
	LoadNomadData(pNomad, NULL);
	// note: the order of call is important
	// TraverseDirs("*.mp3") && LoadNomadData must be called first
	TraverseDirs("*.m3u");
	CreateNomadPlaylists(pNomad);
	Match();
}

wxDirTraverseResult PlaylistMatcher::OnFile(const wxString& strFileSpec)
{
	if (strFileSpec.Last() == 'u') 
	{
		OnPlayListFile(strFileSpec);
		return wxDIR_CONTINUE;
	}
	return Matcher::OnFile(strFileSpec);
}	//	OnFile

//	OnPlayListFile: loads the playlist file
void PlaylistMatcher::OnPlayListFile(const wxString& strFileSpec)
{
	int nPos = strFileSpec.Find(wxFILE_SEP_PATH, true); // reverse find
	// to do: put this in playlist??
	nPos++;
	wxString strName(strFileSpec.Mid(nPos, strFileSpec.Length() - 4 - nPos));

	map<wxString, Playlist*>::iterator iterMap;
	iterMap = m_mapPCPlaylist.find(strName); 
	Playlist* pPlaylist = NULL;
	if (iterMap != m_mapPCPlaylist.end()) 
	{
		wxLogMessage("Duplicate playlist ignored: %s ", (const char*)strFileSpec);
	}
	else
	{
		pPlaylist = new Playlist;
		pPlaylist->SetName(strName);
		pPlaylist->SetFileSpec(strFileSpec);
		pPlaylist->ParseFile(m_pTrackCache->GetMapPCCache());
		pPlaylist->TransformTracks(m_mapNomadTrack);
		m_mapPCPlaylist[strName] = pPlaylist;
	}
}	//	OnPlayListFile

// CreateNomadPlaylists: copys playlists from Nomad and gets the 
// transfer tracks
void PlaylistMatcher::CreateNomadPlaylists(const Nomad* pNomad)
{
	map<wxString, Playlist*>::const_iterator iter; 
	Playlist* pPlaylist;
	for (iter = pNomad->GetMapPlaylist().begin(); 
		iter != pNomad->GetMapPlaylist().end(); iter++)
	{
		pPlaylist = new Playlist(*(iter->second));
		m_mapNomadPlaylist[iter->first] = pPlaylist;
		pPlaylist->TransformTracks(m_mapPCTrack);
	}
}


void PlaylistMatcher::Match()
{
	map<wxString, Playlist*>::iterator iterPC, iterNomad; 
	for (iterPC = m_mapPCPlaylist.begin(); iterPC != m_mapPCPlaylist.end(); iterPC++) 
	{
		iterNomad = m_mapNomadPlaylist.find(iterPC->second->GetName().Lower()); 
		if (iterNomad != m_mapNomadPlaylist.end())
		{
			if (!iterPC->second->HasSameTracks(iterNomad->second))
			{
				m_listDiffPlaylist.push_back(
					PlaylistPair(iterPC->second, iterNomad->second));
			}
			else delete iterNomad->second;	
			m_mapNomadPlaylist.erase(iterNomad);
		}	
		else	// not found on nomad
		{
			m_listPCExtraPlaylist.push_back(iterPC->second);
		}
	}	// end for
}	 //	MatchPlaylists


void PlaylistMatcher::CreateActions(ActionTable& table) const
{
	unsigned int n;
	PlaylistAction *pAction, *pAction2;

	table.RemoveActions();
	for (n = 0; n < m_listPCExtraPlaylist.size(); n++)
	{
		pAction = new PlaylistAction(m_config);
		pAction->SetPlaylist(m_listPCExtraPlaylist[n]);
		pAction->SetDefault();
		table.AddAction(pAction);
	}

	PlaylistMap::const_iterator iterMap; 
	for (iterMap = m_mapNomadPlaylist.begin(); 
		iterMap != m_mapNomadPlaylist.end(); iterMap++) 
	{
		pAction = new PlaylistAction(m_config);
		pAction->SetPlaylist(iterMap->second);
		pAction->SetDefault();
		table.AddAction(pAction);
	}
	for (n = 0; n < m_listDiffPlaylist.size(); n++)
	{
		pAction = new PlaylistAction(m_config);
		pAction->SetPlaylist(m_listDiffPlaylist[n].m_pPC);
		pAction2 = new PlaylistAction(m_config);
		pAction2->SetPlaylist(m_listDiffPlaylist[n].m_pNomad);
		pAction2->SetDependAction(pAction);
		pAction->SetDependAction(pAction2);
		pAction->SetDefault();
		pAction2->SetDefault();
		table.AddAction(pAction);
		table.AddAction(pAction2);
	}
	
	table.Sort(c_nIconCol);
}	// end CreateActions



