#ifndef _TRACK_H_INCLUDED_
#define _TRACK_H_INCLUDED_

#include "wx/laywin.h"

enum trackType
{
	typeMP3,
	typeWMA,
};

// trackAttribStr is used to index track string attributes
enum trackAttribStr
{
	taFileSpec = 0,
	taTitle,
	taArtist,
	taAlbum,
	taGenre,
	taCodec,
	taMaxStrAttribs
};

// trackAttribStr is used to index track number attributes
enum trackAttribNum
{
	taFileSize = taMaxStrAttribs,
	taLength,
	taTrackNum,
	taYear,
	taTotalAttribs
};


// Used for label matching algorithm
const int c_nMaxSimLevel = 6;
const int c_nSameLabelsNoCase = 7;

class EzGrid;
class Config;

// Track controls everything to do with tracks both on PC and on Nomad
class Track
{
public:
	Track();
	~Track();
	bool Load(const wxString& strFileSpec);
	const char* Parse(const char* pCurr, const char* pEnd);
	int GetFileSize() const
	{
		return m_nFileSize;
	}
	void SetFileSize(int nFileSize)
	{
		m_nFileSize = nFileSize;
	}
	int GetTrackLength() const
	{
		return m_nTrackLength;
	}
	void SetTrackLength(int nTrackLength)
	{
		m_nTrackLength = nTrackLength;
	}
	int GetYear() const
	{
		return m_nYear;
	}
	void SetYear(int nYear)
	{
		if (nYear >= 0) m_nYear = nYear;
	}
	int GetTrackNum() const
	{
		return m_nTrackNum;
	}
	void SetTrackNum(int nTrackNum)
	{
		if (nTrackNum >= 0) m_nTrackNum = nTrackNum;
	}
	trackType GetType() const
	{
		if (m_strAttrib[taCodec].CmpNoCase("mp3") == 0) return typeMP3;
		return typeWMA;
	}
	wxString GetAttribAsStr(int n) const;
	wxString GetFilledAttrib(int n) const;
	const wxString& GetAttrib(trackAttribStr attrib) const
	{
		wxASSERT(attrib >= taFileSpec && attrib < taMaxStrAttribs);
		return m_strAttrib[attrib];
	}
	void SetAttrib(trackAttribStr attrib, const wxString& strAttrib);
	unsigned int GetItemID() const
	{
		return m_nItemID;
	}
	void SetItemID(unsigned int nItemID)
	{
		m_nItemID = nItemID;
	}
	wxString GetValue(int nCol) const;
	void Dump(wxFile& fileOut) const;
	wxString GetKey(const Config* pConfig = NULL) const;
	wxString GetShortKey(int nAttribLen) const;
	bool SameSize(const Track* pTrack) const;
	bool SameSizeAndLen(const Track* pTrack) const;
	bool IsSimiliar(const Track* pTrack, int nSimilarityLevel) const;
	bool SameKey(const Track* pTrack) const;
	bool EqualNoCase(const Track* pTrack) const;
	bool IsNomad() const
	{
		return m_strAttrib[taFileSpec].IsEmpty();
	}
	wxFileName GenerateFileSpec(const wxString& strPrimaryDir) const;
	void SetTag(const wxString& strFileSpec);
	void SetTag(const Track* pTrack);
	wxString GetDisplayName() const;
	bool IsFaulty() const
	{
		return m_bFaulty;
	}
	void SetFaulty(bool bFaulty)
	{
		m_bFaulty = bFaulty;
	}
	bool MissingTag() const
	{
		return m_bMissingTag;
	}
	time_t GetTimeFileModified() const
	{
		return m_timeFileModified;
	}
	bool ColIsLessThan(int nCol, const Track* pTrack) const;
private:
	static int m_nCountMissingTags;
	wxString m_strAttrib[taMaxStrAttribs];
	int m_nFileSize;
	int m_nTrackLength;
	int m_nTrackNum;
	int m_nYear;
	unsigned int m_nItemID;		// Stores the nomad trackID	
	bool m_bFaulty;
	// Stores the file modified timestamp. Used for caching
	time_t m_timeFileModified;	
	bool m_bMissingTag;
};


#endif // _TRACK_H_INCLUDED_
