#include "Precompiled.h"
#include "TrackCache.h"
#include "Track.h"


TrackCache::~TrackCache()
{
	TrackMap::iterator iterMap; 
	for (iterMap = m_mapPCCache.begin(); iterMap != m_mapPCCache.end(); iterMap++) 
		delete iterMap->second;
}

void TrackCache::WriteCacheFile(const wxString& strFileSpec)
{
	if (m_listPCAll.size()== 0) return;	// user hasn't done search
	wxFile fileOut;
	if (!fileOut.Open(strFileSpec, wxFile::write))
	{
		wxLogMessage("Can't write to cache file %s", (const char*)strFileSpec);
		wxLogMessage(wxSysErrorMsg(wxSysErrorCode()));
		return;
	}
	// note: we don't write from m_mapPCCache, because we want to eleminate the
	// non-hits
//	wxLogMessage("Cache %d, Hits %d", m_mapPCCache.size(), m_listPCAll.size());
	unsigned int n;
	for (n = 0; n < m_listPCAll.size(); n++)
	{
		m_listPCAll[n]->Dump(fileOut);
	}
	fileOut.Close();
}

void TrackCache::ReadCacheFile(const wxString& strFileSpec)
{
	wxFile fileIn;
	if (!fileIn.Open(strFileSpec, wxFile::read))
	{
		wxLogMessage("Can't open cache file %s", (const char*)strFileSpec);
		wxLogMessage(wxSysErrorMsg(wxSysErrorCode()));
		return;
	}
	char* pBuff = new char[fileIn.Length()];
	fileIn.Read((void*)pBuff, fileIn.Length());
	const char*	pCurr = pBuff;
	const char*	pEnd = pBuff+fileIn.Length();
	Track* pTrack;
	pTrack = new Track;
	while ((pCurr = pTrack->Parse(pCurr, pEnd)))
	{
		m_mapPCCache[pTrack->GetAttrib(taFileSpec)] = pTrack;
		pTrack = new Track;
	} 
	delete pTrack;
	fileIn.Close();
	delete [] pBuff;
}	 // ReadCacheFile

// GetFile: retrieves the track from the cache, if found, otherwise
// creates a new track and loads it
Track* TrackCache::GetFile(const wxString& strFileSpec)
{
	TrackMap::iterator iterMap;
	iterMap = m_mapPCCache.find(strFileSpec); 
	Track* pTrack = NULL;
	time_t time;
	if (iterMap != m_mapPCCache.end()) 
	{
		time = wxFileModificationTime(strFileSpec);
		pTrack = iterMap->second;
		if (time != pTrack->GetTimeFileModified()) 
		{
			// the cached track has been modified, so delete it
			delete pTrack;
			pTrack = NULL;
		}
	}
	if (pTrack == NULL) 
	{
		pTrack = new Track;
		pTrack->Load(strFileSpec);
		m_mapPCCache[strFileSpec] = pTrack;
	}
	m_listPCAll.push_back(pTrack);
	return pTrack;
}	// GetFile
