#ifndef _TRACKMATCHER_H_INCLUDED_
#define _TRACKMATCHER_H_INCLUDED_

#include <wx/dir.h>

using namespace std;


class Config;
class Track;
class Nomad;
class EzGrid;
class Action;
class ActionTable;
class TrackCache;

typedef vector<Action*> ActionList;
typedef vector<Track*> TrackList;
typedef map<wxString, Track*> TrackMap; 

class TrackPair
{
public:
	TrackPair(Track* pTrackPC, Track* pTrackNomad) : m_pTrackPC(pTrackPC),
		m_pTrackNomad(pTrackNomad) {}
	Track* m_pTrackPC;
	Track* m_pTrackNomad;
};

// Matcher: abstract base class for TrackMatcher & PlaylistMatcher
class Matcher : public wxDirTraverser
{
public:
	Matcher(TrackCache* pTrackCache, const Config& config) : 
		m_config(config), m_pTrackCache(pTrackCache)
	{
	}
	virtual ~Matcher() {}
	virtual void RemoveAll();
	virtual void Run(Nomad* pNomad) = 0;
	virtual wxDirTraverseResult OnFile(const wxString& strFilename);
	virtual wxDirTraverseResult OnDir(const wxString& strDirname);
	virtual void CreateActions(ActionTable& table) const = 0;
protected:
	void TraverseDirs(const wxString& strExt);
	void LoadNomadData(Nomad* pNomad, const Config* pConfig);
	virtual void Match() = 0;
	// maps indexed by key (Title/artist/album) 
	TrackMap m_mapPCTrack;
	// m_mapExtPCTrack is indexed by key (Title/artist/album/genre/tracknum, etc) 
	TrackMap m_mapExtPCTrack;
	TrackMap m_mapNomadTrack;
	// store list of PC duplicates
	TrackList m_listDup;		
	const Config& m_config;
	TrackCache* m_pTrackCache;
};


class TrackMatcher : public Matcher
{
public:
	TrackMatcher(TrackCache* pTrackCache, const Config& config) : 
		Matcher(pTrackCache, config)
	{
	}
	virtual ~TrackMatcher() {}
	virtual void RemoveAll();
	virtual void Run(Nomad* pNomad);
	virtual void CreateActions(ActionTable& table) const;
private:
	virtual void Match();
	void FindLabelMatches(vector<TrackPair>& listPair, int nSimLevel);
	TrackList m_listPCExtra;	// stores tracks not found on nomad
	// stores similiar tracks with slightly different labels
	vector<TrackPair> m_listRelabeled;	
	// stores tracks with same label, but different file sizes
	vector<TrackPair> m_listDiffSize;	
};


#endif // _TRACKMATCHER_H_INCLUDED_
